@@ -1,4 +1,4 @@
-/* $Id: codebuild.c 1955 2007-02-28 17:07:04Z coudercd $ */
+/* $Id: codebuild.c 1989 2007-05-23 20:25:50Z coudercd $ */
 
 /*
  * Copyright (c) 2005-2006 Damien Couderc
@@ -468,6 +468,11 @@ bool c_code_builder(code_bld_t *pcb) {
 	if (pcb->header != NULL) {
 		code_logger(tfp, lfp, "/* main header to test */\n");
 		code_logger(tfp, lfp, CODE_C_HDR, pcb->header);
+	} else {
+		if (pcb->procedure != NULL) {
+			/* prototype */
+			code_logger(tfp, lfp, CODE_C_DECL, pcb->procedure);
+		}
 	}
 
 	/* main proc */
