@@ -1,4 +1,4 @@
-/* $Id: pmk_ctype.h,v 1.3 2004/07/11 20:33:34 mipsator Exp $ */
+/* $Id: pmk_ctype.h 1444 2005-08-06 15:08:54Z mipsator $ */
 
 /*
  * Copyright (c) 2003 Damien Couderc
@@ -39,14 +39,17 @@
 
 #include <ctype.h>
 
-#include "compat.h"
+#include "config.h"
+
 
 /*
 	isblank() function
 */
 
 #ifndef HAVE_ISBLANK
-#include "isblank.h"
+
+int isblank(int);
+
 #endif /* HAVE_ISBLANK */
 
 #endif /* _PMK_CTYPE_H_ */
