@@ -1,7 +1,7 @@
-/* $Id: pmk_libgen.h,v 1.3 2004/07/11 20:33:34 mipsator Exp $ */
+/* $Id: pmk_libgen.h 1444 2005-08-06 15:08:54Z mipsator $ */
 
 /*
- * Copyright (c) 2003 Damien Couderc
+ * Copyright (c) 2003-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,16 +37,22 @@
 #ifndef _PMK_LIBGEN_H_
 #define _PMK_LIBGEN_H_
 
-#include "compat.h"
+#include "config.h"
+
 
 /*
 	basename() and dirname() functions
 */
 
 #ifdef HAVE_LIBGEN_H
+
 #include <libgen.h>
-#else
-#include "libgen.h"
+
+#else /* HAVE_LIBGEN_H */
+
+char *basename(const char *);
+char *dirname(const char *);
+
 #endif /* HAVE_LIBGEN_H */
 
 #endif /* _PMK_LIBGEN_H_ */
