@@ -1,7 +1,7 @@
-/* $Id: pmk_stdbool.h,v 1.4 2004/07/11 20:33:34 mipsator Exp $ */
+/* $Id: pmk_stdbool.h 1496 2005-10-23 16:39:56Z mipsator $ */
 
 /*
- * Copyright (c) 2003 Damien Couderc
+ * Copyright (c) 2003-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,16 +37,52 @@
 #ifndef _PMK_STDBOOL_H_
 #define _PMK_STDBOOL_H_
 
-#include "compat.h"
+#include "config.h"
+
 
 /*
 	boolean types
 */
 
 #ifdef HAVE_STDBOOL_H
+
 #include <stdbool.h>
-#else
-#include "stdbool.h"
+
+#else /* HAVE_STDBOOL_H */
+
+/*
+	Sun Forte 5.4 C compiler and previous versions give out a
+	"warning: _Bool is a keyword in ISO C99"
+	message when we use typedef. In addition, every Sun Pro C compiler
+	which conforms to ANSI C (and therefore sets __STDC__ to 1) shows the
+	same behaviour. We avoid this by defining _Bool in a macro in these
+	cases.
+*/
+
+#ifndef HAVE__BOOL
+#	if defined __SUNPRO_C && (__SUNPRO_C < 0x550 || __STDC__ == 1)
+#		define _Bool unsigned char
+#	else /* __SUNPRO_C && (__SUNPRO_C < 0x550 || __STDC__ == 1) */
+typedef unsigned char _Bool;
+#	endif /* __SUNPRO_C && (__SUNPRO_C < 0x550 || __STDC__ == 1) */
+#endif /* HAVE__BOOL */
+
+#ifndef bool
+#define bool	_Bool
+#endif /* bool */
+
+#ifndef true
+#define true	1
+#endif /* true */
+
+#ifndef false
+#define false	0
+#endif /* false */
+
+#ifndef __bool_true_false_are_defined
+#define __bool_true_false_are_defined 1
+#endif /* __bool_true_false_are_defined */
+
 #endif /* HAVE_STDBOOL_H */
 
 #endif /* _PMK_STDBOOL_H_ */
