@@ -1,7 +1,7 @@
-/* $Id: pmk_string.h,v 1.6 2004/12/26 15:02:46 mipsator Exp $ */
+/* $Id: pmk_string.h 1449 2005-08-10 20:30:35Z mipsator $ */
 
 /*
- * Copyright (c) 2003 Damien Couderc
+ * Copyright (c) 2003-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,44 +39,46 @@
 
 #include <string.h>
 
-#include "compat.h"
+#include "config.h"
 #include "pmk_stdbool.h"
 
+
+#ifndef HAVE_STRDUP
+
+/*
+	strdup() function
+*/
+
+char	*strdup(const char *);
+
+#endif /* HAVE_STRDUP */
+
+
+#ifndef HAVE_STRLCPY
+
 /*
 	strlcpy() function
-
-	Systems known to have compatible strlcpy in string.h :
-		OpenBSD		since 2.4
-		FreeBSD		since 3.3
-		NetBSD		since 1.5
-		SCO OpenServer	since 5.0.6
-		SunOs		since 5.8
-		MacOSX
-
-	Systems without strlcpy in string.h :
-		AIX		(4.3)
-		glibc based	(at least before 4 Mar 2002)
-
-	Note: Feel free to send us a notice if your system is not
-		listed here but should be.
 */
 
-#ifndef HAVE_STRLCPY
-#include "strlcpy.h"
-#endif
+size_t	 strlcpy(char *, const char *, size_t);
+
+#endif /* HAVE_STRLCPY */
+
+
+#ifndef HAVE_STRLCAT
 
 /*
 	strlcat() function
 */
 
-#ifndef HAVE_STRLCAT
-#include "strlcat.h"
-#endif
+size_t	 strlcat(char *, const char *, size_t);
+
+#endif /* HAVE_STRLCAT */
+
+
+bool	 snprintf_b(char *, size_t, const char *, ...);
+bool	 strlcat_b(char *, const char *, size_t);
+bool	 strlcpy_b(char *, const char *, size_t);
 
 #endif /* _PMK_STRING_H_ */
 
-
-bool	snprintf_b(char *, size_t, const char *, ...);
-bool	strlcat_b(char *, const char *, size_t);
-bool	strlcpy_b(char *, const char *, size_t);
-
