@@ -1,7 +1,7 @@
-/* $Id: pmk_sys_types.h,v 1.4 2004/07/11 20:33:34 mipsator Exp $ */
+/* $Id: pmk_sys_types.h 1444 2005-08-06 15:08:54Z mipsator $ */
 
 /*
- * Copyright (c) 2003-2004 Damien Couderc
+ * Copyright (c) 2003-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,7 +38,8 @@
 
 #include <sys/types.h>
 
-#include "compat.h"
+#include "config.h"
+
 
 /*
 	blkcnt_t type
