@@ -1,7 +1,7 @@
-/* $Id: pmk_unistd.h,v 1.2 2004/07/11 20:33:34 mipsator Exp $ */
+/* $Id: pmk_unistd.h 1444 2005-08-06 15:08:54Z mipsator $ */
 
 /*
- * Copyright (c) 2003 Damien Couderc
+ * Copyright (c) 2003-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,14 +39,17 @@
 
 #include <unistd.h>
 
-#include "compat.h"
+#include "config.h"
+
 
 /*
 	mkstemps() function
 */
 
 #ifndef HAVE_MKSTEMPS
-#include "mkstemps.h"
+
+int	mkstemps(char *, int);
+
 #endif /* HAVE_MKSTEMPS */
 
 #endif /* _PMK_UNISTD_H_ */
