@@ -1,4 +1,4 @@
-/* $Id: detect_cpu.c,v 1.28 2005/01/26 17:03:34 mipsator Exp $ */
+/* $Id: detect_cpu.c 1530 2006-01-12 20:07:40Z mipsator $ */
 
 /*
  * Copyright (c) 2004-2005 Damien Couderc
@@ -34,7 +34,6 @@
  */
 
 
-#include <stdio.h>
 #include <stdlib.h>
 /* include it first as if it was <sys/types.h> - this will avoid errors */
 #include "compat/pmk_sys_types.h"
@@ -42,6 +41,7 @@
 #include <errno.h>
 
 #include "compat/pmk_string.h"
+#include "compat/pmk_stdio.h"
 #include "common.h"
 #include "detect_cpu.h"
 #include "detect_cpu_asm.h"
@@ -49,44 +49,49 @@
 
 /***************
  * common code *
- ***************/
+ ***********************************************************************/
 
 
 /* config tools data file keyword */
 prskw	kw_pmkcpu[] = {
-	{"LIST_ARCH_EQUIV",		LIST_ARCH_EQUIV,	PRS_KW_CELL},
-	{"LIST_X86_CPU_VENDOR",		LIST_X86_CPU_VENDOR,	PRS_KW_CELL},
-	{"LIST_X86_CPU_MODEL",		LIST_X86_CPU_MODEL,	PRS_KW_CELL},
-	{"LIST_X86_CPU_CLASS",		LIST_X86_CPU_CLASS,	PRS_KW_CELL},
-	{"LIST_ALPHA_CPU_CLASS",	LIST_ALPHA_CPU_CLASS,	PRS_KW_CELL},
-	{"LIST_IA64_CPU_CLASS",		LIST_IA64_CPU_CLASS,	PRS_KW_CELL}
+	{"LIST_ARCH_EQUIV",			LIST_ARCH_EQUIV,		PRS_KW_CELL,	PRS_TOK_NULL,	NULL},
+	{"LIST_X86_CPU_VENDOR",		LIST_X86_CPU_VENDOR,	PRS_KW_CELL,	PRS_TOK_NULL,	NULL},
+	{"LIST_X86_CPU_MODEL",		LIST_X86_CPU_MODEL,		PRS_KW_CELL,	PRS_TOK_NULL,	NULL},
+	{"LIST_X86_CPU_CLASS",		LIST_X86_CPU_CLASS,		PRS_KW_CELL,	PRS_TOK_NULL,	NULL},
+	{"LIST_ALPHA_CPU_CLASS",	LIST_ALPHA_CPU_CLASS,	PRS_KW_CELL,	PRS_TOK_NULL,	NULL},
+	{"LIST_IA64_CPU_CLASS",		LIST_IA64_CPU_CLASS,	PRS_KW_CELL,	PRS_TOK_NULL,	NULL}
 };
 size_t	nbkwc = sizeof(kw_pmkcpu) / sizeof(prskw);
 
 
 arch_cell	arch_tab[] = {
-	{"x86_32",	PMK_ARCH_X86_32},
-	{"x86_64",	PMK_ARCH_X86_64},
-	{"sparc32",	PMK_ARCH_SPARC},
-	{"sparc64",	PMK_ARCH_SPARC64},
-	{"ia_64",	PMK_ARCH_IA_64},
-	{"ppc",		PMK_ARCH_PPC},
-	{"alpha",	PMK_ARCH_ALPHA},
-	{"m68k",	PMK_ARCH_M68K},
+	{"x86_32",		PMK_ARCH_X86_32},
+	{"x86_64",		PMK_ARCH_X86_64},
+	{"sparc32",		PMK_ARCH_SPARC},
+	{"sparc64",		PMK_ARCH_SPARC64},
+	{"ia_64",		PMK_ARCH_IA_64},
+	{"ppc",			PMK_ARCH_PPC},
+	{"alpha",		PMK_ARCH_ALPHA},
+	{"m68k",		PMK_ARCH_M68K},
 	{"parisc_64",	PMK_ARCH_PARISC},
-	{"vax",		PMK_ARCH_VAX}
+	{"vax",			PMK_ARCH_VAX}
 };
 size_t	nbarch = sizeof(arch_tab) / sizeof(arch_cell);
 
 
-/*
+/**************
+ * seek_key() *
+ ***********************************************************************
+ DESCR
 	seek parse data key
 
+ IN
 	pdata : parse data structure
 	token : key token to find
 
-	returns : key data or NULL
-*/
+ OUT
+	key data or NULL
+ ***********************************************************************/
 
 void *seek_key(prsdata *pdata, int token) {
 	prscell		*pcell;
@@ -103,13 +108,19 @@ void *seek_key(prsdata *pdata, int token) {
 	return(NULL);
 }
 
-/*
+
+/********************
+ * parse_cpu_data() *
+ ***********************************************************************
+ DESCR
 	parse cpu data file
 
+ IN
 	fname : data file name
 
-	returns : prsdata structure
-*/
+ OUT
+	prsdata structure
+ ***********************************************************************/
 
 prsdata *parse_cpu_data(char *fname) {
 	FILE		*fp;
@@ -146,13 +157,18 @@ prsdata *parse_cpu_data(char *fname) {
 }
 
 
-/*
-	check the cpu architecture 
+/********************
+ * check_cpu_arch() *
+ ***********************************************************************
+ DESCR
+	check the cpu architecture
 
-	uname_m : uname machine string 
+ IN
+	uname_m : uname machine string
 
-	returns:  cpu architecture string
-*/
+ OUT
+	cpu architecture string
+ ***********************************************************************/
 
 char *check_cpu_arch(char *uname_m, prsdata *pdata) {
 	char		*pstr = NULL;
@@ -172,13 +188,19 @@ char *check_cpu_arch(char *uname_m, prsdata *pdata) {
 	return(strdup(pstr)); /* no check needed, NULL will be automatically returned */
 }
 
-/*
+
+/*********************
+ * arch_name_to_id() *
+ ***********************************************************************
+ DESCR
 	convert arch name to id
 
+ IN
 	arch_name : architecture string
 
-	returns:  architecture identifier
-*/
+ OUT
+	architecture identifier
+ ***********************************************************************/
 
 unsigned char arch_name_to_id(char *arch_name) {
 	unsigned int	i;
@@ -196,17 +218,23 @@ unsigned char arch_name_to_id(char *arch_name) {
 	return(id);
 }
 
-/*
+
+/******************
+ * arch_wrapper() *
+ ***********************************************************************
+ DESCR
 	architecture wrapper
 
+ IN
 	pdata : parsing data structure
 	arch_name : architecture name
 
-	returns: hash table with values or NULL
-*/
+ OUT
+	hash table with values or NULL
+ ***********************************************************************/
 
 htable *arch_wrapper(prsdata *pdata, char *arch_name) {
-	htable		*pht;
+	htable			*pht;
 	unsigned char	 arch_id;
 
 #if defined(ARCH_X86_32) || defined(ARCH_X86_64)
@@ -233,9 +261,8 @@ htable *arch_wrapper(prsdata *pdata, char *arch_name) {
 				errorf("unable to get x86 cpuid data.");
 				return(NULL);
 			}
-		
-			pcell->stdvendor = x86_get_std_cpu_vendor(pdata,
-							pcell->vendor);
+
+			pcell->stdvendor = x86_get_std_cpu_vendor(pdata, pcell->vendor);
 			if (x86_set_cpu_data(pdata, pcell, pht) == false) {
 				errorf("failed to record cpu data.");
 				return(NULL);
@@ -280,7 +307,7 @@ htable *arch_wrapper(prsdata *pdata, char *arch_name) {
 
 /****************
  * x86 specific *
- ****************/
+ ***********************************************************************/
 
 #if defined(ARCH_X86_32) || defined(ARCH_X86_64)
 
@@ -342,11 +369,18 @@ size_t nb_feat_extreg = sizeof(x86_cpu_feat_extreg) / sizeof(x86_cpu_feature);
  functions
 ****/
 
-/*
+/***********************
+ * x86_cpu_cell_init() *
+ ***********************************************************************
+ DESCR
 	initialise x86 cpu cell
 
-	returns : cpu cell or NULL
-*/
+ IN
+	NONE
+
+ OUT
+	cpu cell or NULL
+ ***********************************************************************/
 
 x86_cpu_cell *x86_cpu_cell_init(void) {
 	x86_cpu_cell	*pcell;
@@ -367,13 +401,19 @@ x86_cpu_cell *x86_cpu_cell_init(void) {
 	return(pcell);
 }
 
-/*
+
+/**************************
+ * x86_cpu_cell_destroy() *
+ ***********************************************************************
+ DESCR
 	free x86 cpu cell structure
 
-	pcell : structure to free
+ IN
+	pcell :	structure to free
 
-	returns : -
-*/
+ OUT
+	NONE
+ ***********************************************************************/
 
 void x86_cpu_cell_destroy(x86_cpu_cell *pcell) {
 	if (pcell->vendor != NULL)
@@ -385,13 +425,19 @@ void x86_cpu_cell_destroy(x86_cpu_cell *pcell) {
 	free(pcell);
 }
 
-/*
+
+/****************************
+ * x86_get_std_cpu_vendor() *
+ ***********************************************************************
+ DESCR
 	get (pmk) vendor identifier based on data from cpuid
 
-	pdata : parsung structure
+ IN
+	pdata :	parsing structure
 
-	returns: cpu vendor string
-*/
+ OUT
+	cpu vendor string
+ ***********************************************************************/
 
 char *x86_get_std_cpu_vendor(prsdata *pdata, char *civendor) {
 	char		*vendor = NULL;
@@ -418,19 +464,24 @@ char *x86_get_std_cpu_vendor(prsdata *pdata, char *civendor) {
 }
 
 
-/*
+/************************
+ * x86_get_cpuid_data() *
+ ***********************************************************************
+ DESCR
 	gather x86 cpuid data
 
-	cell : x86 cpu cell
+ IN
+	cell :	x86 cpu cell
 
-	returns : true on success else false
-*/
+ OUT
+	boolean
+ ***********************************************************************/
 
 bool x86_get_cpuid_data(x86_cpu_cell *cell) {
-	char		 feat_str[TMP_BUF_LEN] = "";
+	char			 feat_str[TMP_BUF_LEN] = "";
 	unsigned int	 i;
-	uint32_t	 buffer[13],
-			 extlevel;
+	uint32_t		 buffer[13],
+					 extlevel;
 
 	if (x86_check_cpuid_flag() == 0) {
 		/* no cpuid flag => 386 or old 486 */
@@ -469,19 +520,19 @@ bool x86_get_cpuid_data(x86_cpu_cell *cell) {
 	/* processing feature register 1 */
 	for (i = 0 ; i < nb_feat_reg1 ; i++) {
 		if ((x86_cpu_reg_edx & x86_cpu_feat_reg1[i].mask) != 0) {
-			strlcat(feat_str, x86_cpu_feat_reg1[i].descr,
-					sizeof(feat_str)); /* no check */
-			if (strlcat_b(feat_str, " ", sizeof(feat_str)) == false)
+			strlcat(feat_str, x86_cpu_feat_reg1[i].descr, sizeof(feat_str)); /* no check */
+			if (strlcat_b(feat_str, " ", sizeof(feat_str)) == false) {
 				return(false);
+			}
 		}
 	}
 	/* processing feature register 2 */
 	for (i = 0 ; i < nb_feat_reg2 ; i++) {
 		if ((x86_cpu_reg_edx & x86_cpu_feat_reg2[i].mask) != 0) {
-			strlcat(feat_str, x86_cpu_feat_reg2[i].descr,
-					sizeof(feat_str)); /* no check */
-			if (strlcat_b(feat_str, " ", sizeof(feat_str)) == false)
+			strlcat(feat_str, x86_cpu_feat_reg2[i].descr, sizeof(feat_str)); /* no check */
+			if (strlcat_b(feat_str, " ", sizeof(feat_str)) == false) {
 				return(false);
+			}
 		}
 	}
 
@@ -496,11 +547,10 @@ bool x86_get_cpuid_data(x86_cpu_cell *cell) {
 		/* processing extended feature register */
 		for (i = 0 ; i < nb_feat_extreg ; i++) {
 			if ((x86_cpu_reg_edx & x86_cpu_feat_extreg[i].mask) != 0) {
-				strlcat(feat_str,
-						x86_cpu_feat_extreg[i].descr,
-						sizeof(feat_str)); /* no check */
-				if (strlcat_b(feat_str, " ", sizeof(feat_str)) == false)
+				strlcat(feat_str, x86_cpu_feat_extreg[i].descr, sizeof(feat_str)); /* no check */
+				if (strlcat_b(feat_str, " ", sizeof(feat_str)) == false) {
 					return(false);
+				}
 			}
 		}
 	}
@@ -530,8 +580,9 @@ bool x86_get_cpuid_data(x86_cpu_cell *cell) {
 
 		buffer[12] = 0;	/* terminate string */
 		cell->cpuname = strdup((char *) buffer);
-		if (cell->cpuname == NULL)
+		if (cell->cpuname == NULL) {
 			return(false);
+		}
 	} else {
 		cell->cpuname = NULL;
 	}
@@ -539,19 +590,25 @@ bool x86_get_cpuid_data(x86_cpu_cell *cell) {
 	return(true);
 }
 
-/*
+
+/**********************
+ * x86_set_cpu_data() *
+ ***********************************************************************
+ DESCR
 	record cpu data in hash table
 
-	pdata : parsing data
-	pcell : x86 cpu data cell
-	pht : storage hash table
+ IN
+	pdata :	parsing data
+	pcell :	x86 cpu data cell
+	pht :	storage hash table
 
-	returns : true on succes else false
-*/
+ OUT
+	boolean
+ ***********************************************************************/
 
 bool x86_set_cpu_data(prsdata *pdata, x86_cpu_cell *pcell, htable *pht) {
 	char	 buffer[TMP_BUF_LEN],
-		*pstr;
+			*pstr;
 	htable	*phtbis;
 
 	if (snprintf_b(buffer, sizeof(buffer), "%u", pcell->family) == false)
@@ -650,7 +707,7 @@ bool x86_set_cpu_data(prsdata *pdata, x86_cpu_cell *pcell, htable *pht) {
 
 /******************
  * alpha specific *
- ******************/
+ ***********************************************************************/
 
 #if defined(ARCH_ALPHA)
 
@@ -664,20 +721,29 @@ alpha_cpu_feature	alpha_cpu_feat[] = {
 };
 int nb_feat = sizeof(alpha_cpu_feat) / sizeof(alpha_cpu_feature);
 
-/*
+
+/************************
+ * alpha_set_cpu_data() *
+ ***********************************************************************
+ DESCR
 	set alpha cpu data
 
-	pdata: parsing data structure
-	pht: data storage hash table
-*/
+ IN
+	pdata:	parsing data structure
+	pht:	data storage hash table
+
+ OUT
+	boolean
+ ***********************************************************************/
+
 bool alpha_set_cpu_data(prsdata *pdata, htable *pht) {
-	char		 buffer[16],
-			 feat_str[TMP_BUF_LEN] = "",
-			*pstr;
-	htable		*phtbis;
+	char			 buffer[16],
+					 feat_str[TMP_BUF_LEN] = "",
+					*pstr;
+	htable			*phtbis;
 	unsigned int	 i;
-	uint64_t	 implver,
-			 amask;
+	uint64_t		 implver,
+					 amask;
 
 /*debugf("alpha_set_cpu_data() : end");*/
 	phtbis = (htable *) seek_key(pdata, LIST_ALPHA_CPU_CLASS);
@@ -686,8 +752,7 @@ bool alpha_set_cpu_data(prsdata *pdata, htable *pht) {
 		implver = alpha_exec_implver();
 /*debugf("alpha_set_cpu_data() : implver = '%u'", implver);*/
 
-		if (snprintf_b(buffer, sizeof(buffer), ALPHA_CPU_CLASS_FMT,
-					implver) == false) {
+		if (snprintf_b(buffer, sizeof(buffer), ALPHA_CPU_CLASS_FMT, implver) == false) {
 			return(false);
 		}
 /*debugf("alpha_set_cpu_data() : buffer = '%s'", buffer);*/
@@ -696,8 +761,7 @@ bool alpha_set_cpu_data(prsdata *pdata, htable *pht) {
 			pstr = ALPHA_CPU_UNKNOWN;
 		}
 
-		if (hash_update_dup(pht, PMKCONF_HW_ALPHA_CPU_CLASS,
-						pstr) == HASH_ADD_FAIL) {
+		if (hash_update_dup(pht, PMKCONF_HW_ALPHA_CPU_CLASS, pstr) == HASH_ADD_FAIL) {
 			return(false);
 		}
 
@@ -707,16 +771,15 @@ bool alpha_set_cpu_data(prsdata *pdata, htable *pht) {
 		/* processing feature mask */
 		for (i = 0 ; i < nb_feat ; i++) {
 			if ((amask & alpha_cpu_feat[i].mask) != 0) {
-				strlcat(feat_str, alpha_cpu_feat[i].descr,
-						sizeof(feat_str)); /* no check */
-				if (strlcat_b(feat_str, " ", sizeof(feat_str)) == false)
+				strlcat(feat_str, alpha_cpu_feat[i].descr, sizeof(feat_str)); /* no check */
+				if (strlcat_b(feat_str, " ", sizeof(feat_str)) == false) {
 					return(false);
+				}
 			}
 		}
 
 		/* save feature string */
-		if (hash_update_dup(pht, PMKCONF_HW_ALPHA_CPU_FEATURES,
-				feat_str) == HASH_ADD_FAIL) {
+		if (hash_update_dup(pht, PMKCONF_HW_ALPHA_CPU_FEATURES, feat_str) == HASH_ADD_FAIL) {
 			return(false);
 		}
 
@@ -725,15 +788,15 @@ bool alpha_set_cpu_data(prsdata *pdata, htable *pht) {
 
 
 /*debugf("alpha_set_cpu_data() : end");*/
-	return(true);		
+	return(true);
 }
 
 #endif /* ARCH_ALPHA */
 
 
-/******************
+/*****************
  * ia64 specific *
- ******************/
+ ***********************************************************************/
 
 #if defined(ARCH_IA64)
 
@@ -749,22 +812,27 @@ size_t nb_feat = sizeof(ia64_cpu_feat) / sizeof(ia64_cpu_feature);
  functions
 ****/
 
-/*
+/*************************
+ * ia64_get_cpuid_data() *
+ ***********************************************************************
+ DESCR
 	gather and save ia64 cpu data
 
-	pdata: parsing data structure
-	pht: storage hash table
+ IN
+	pdata:	parsing data structure
+	pht:	storage hash table
 
-	returns: true on success else false
-*/
+ OUT
+	boolean
+ ***********************************************************************/
 
 bool ia64_get_cpuid_data(prsdata *pdata, htable *pht) {
-	char		 buffer[TMP_BUF_LEN],
-			*pstr;
-	htable		*phtbis;
-	uint64_t	 regbuf[3],
-			 level,
-			 rslt;
+	char			 buffer[TMP_BUF_LEN],
+					*pstr;
+	htable			*phtbis;
+	uint64_t		 regbuf[3],
+					 level,
+					 rslt;
 	unsigned int	 i;
 
 	regbuf[0] = ia64_get_cpuid_register(0);
@@ -866,5 +934,3 @@ bool ia64_get_cpuid_data(prsdata *pdata, htable *pht) {
 
 #endif /* ARCH_IA64 */
 
-
-
