@@ -1,4 +1,4 @@
-/* $Id: func.c 1955 2007-02-28 17:07:04Z coudercd $ */
+/* $Id: func.c 1987 2007-05-12 16:54:49Z coudercd $ */
 
 /*
  * Copyright (c) 2003-2006 Damien Couderc
@@ -939,6 +939,8 @@ bool pmk_check_lib(pmkcmd *cmd, htable *ht, pmkdata *pgd) {
 		return(false);
 	}
 	scb.library = pstr;
+	snprintf(lib_buf, sizeof(lib_buf), "lib%s", scb.library);
+	record_def_data(pgd->htab, lib_buf, NULL);
 	record_def_adv(pgd->htab, TAG_TYPE_LIB, scb.library, NULL, NULL, NULL);
 
 	/* check if a function or more must be searched */
