@@ -1,7 +1,7 @@
-/* $Id: functool.h,v 1.36 2004/12/19 15:18:37 mipsator Exp $ */
+/* $Id: functool.h 1904 2006-10-07 20:39:52Z coudercd $ */
 
 /*
- * Copyright (c) 2003-2004 Damien Couderc
+ * Copyright (c) 2003-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,55 +37,65 @@
 #ifndef _PMK_FUNCTOOL_H_
 #define _PMK_FUNCTOOL_H_
 
-
 #include "common.h"
 #include "hash_tools.h"
 #include "pmk.h"
 #include "premake.h"
 
 
+/*************
+ * constants *
+ **********************************************************************************************/
+
 #define LANG_NAME_LEN	64
 #define COMP_NAME_LEN	64
 #define PRE_NAME_LEN	64
 #define CFLG_NAME_LEN	64
 #define SHFLG_NAME_LEN	64
 
+#define KW_OPT_DEPEND	"DEPEND"
+#define KW_OPT_LANG		"LANG"
+#define KW_OPT_REQUIRED	"REQUIRED"
+
 #define CHAR_VERSION_SEPARATOR	'.'
 
-#define C_FILE_EXT	".c"
-#define TEST_FILE_NAME	TMPDIR "/pmk_XXXXXXXX" C_FILE_EXT
 
+/**********************************
+ * type and structure definitions *
+ ***********************************************************************/
 
 typedef struct {
 	char	name[LANG_NAME_LEN],
-		comp[COMP_NAME_LEN],
-		pre[PRE_NAME_LEN],
-		cflg[CFLG_NAME_LEN],
-		slflg[CFLG_NAME_LEN];
+			comp[COMP_NAME_LEN],
+			pre[PRE_NAME_LEN],
+			cflg[CFLG_NAME_LEN],
+			slflg[CFLG_NAME_LEN];
 } lgdata;
 
 
+/**************
+ * prototypes *
+ ***********************************************************************/
+
 bool	 check_bool_str(char *);
 bool	 invert_bool(bool);
 char	*bool_to_str(bool);
 bool	 get_file_dir_path(char *, char *, char *, int);
-char	*str_to_def(char *);
-char	*build_def_name(char *);
+bool	 find_pattern(dynary *, char *, char *, int);
 bool	 record_def(htable *, char *, bool);
 bool	 record_def_data(htable *, char *, char *);
-bool	 process_def_list(htable *, dynary *);
+bool	 record_def_adv(htable *, int, char *, char *, char *, char *);
+bool	 process_def_list(htable *, dynary *, bool);
 bool	 record_have(htable *, char *, char *);
-bool	 record_val(htable *, char *, char *);
 bool	 label_set(htable *, char *, bool);
 bool	 label_check(htable *, char *);
 bool	 depend_check(htable *, pmkdata *);
 bool	 require_check(htable *);
-lgdata	*check_lang(char *);
-lgdata	*check_lang_comp(char *);
-lgdata	*get_lang(htable *, pmkdata *);
-char	*get_comp_path(htable *, char *);
+char	*get_lang_str(htable *, pmkdata *);
 bool	 check_cfgt_data(pmkdata *);
 bool	 process_required(pmkdata *, pmkcmd *, bool , char *, char *);
-bool	 c_file_builder(char *, size_t, char *, ...);
+bool	 obsolete_string_to_list(htable *, char *);
 
 #endif /* _PMK_FUNCTOOL_H_ */
+
+/* vim: set noexpandtab tabstop=4 softtabstop=4 shiftwidth=4: */
