@@ -1,12 +1,7 @@
-/* $Id: hash.c,v 1.54 2004/12/26 19:32:40 mipsator Exp $ */
+/* $Id: hash.c 1541 2006-02-27 21:11:55Z mipsator $ */
 
 /*
- * Credits for patches :
- *	- Marek Habersack
- */
-
-/*
- * Copyright (c) 2003-2004 Damien Couderc
+ * Copyright (c) 2003-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,6 +34,11 @@
  */
 
 
+/*
+ * Credits for patches :
+ *	- Marek Habersack
+ */
+
 /*******************************************************************
  *                                                                 *
  * Hash-coding functions                                           *
@@ -46,10 +46,10 @@
  *******************************************************************/
 
 
-#include <stdio.h>
 #include <stdlib.h>
 
 #include "hash.h"
+#include "compat/pmk_stdio.h"
 #include "compat/pmk_string.h"
 
 /*#define HASH_DEBUG 1*/
@@ -58,19 +58,24 @@
 #include "common.h"
 #endif
 
-/*
+/******************
+ * hash_compute() *
+ ***********************************************************************
+ DESCR
 	compute hash (perfect hashing)
 
+ IN
 	key : key string
 
-	return : resulting hash
-*/
+ OUT
+	resulting hash
+ ***********************************************************************/
 
 unsigned int hash_compute(char *key, size_t table_size) {
 	unsigned int	len = 0,
-			hash = 0;
+					hash = 0;
 	unsigned char	c;
-	
+
 	c = *key;
 	while (c != '\0') {
 		/* sum all characters with modulo */
@@ -86,39 +91,54 @@ unsigned int hash_compute(char *key, size_t table_size) {
 	return(hash);
 }
 
-/*
+
+/***************
+ * hash_init() *
+ ***********************************************************************
+ DESCR
 	init hash structure of character strings
 
+ IN
 	table_size : number of hash elements
 
-	return : a pointer to the hash structure
-*/
+ OUT
+	 a pointer to the hash structure
+ ***********************************************************************/
 
 htable *hash_init(size_t table_size) {
-	return(hash_init_adv(table_size, (void *(*)(void *))strdup, free, hash_str_append));
+	return(hash_init_adv(table_size, (void *(*)(void *))strdup,
+											free, hash_str_append));
 }
 
-/*
+
+/*******************
+ * hash_init_adv() *
+ ***********************************************************************
+ DESCR
 	init hash structure of objects
 
+ IN
 	table_size : number of hash elements
+	dupfunc : function to duplicate an object
 	freefunc : function to free an object
 	appdfunc : function to create appended object
 
-	return : a pointer to the hash structure
+ OUT
+	a pointer to the hash structure
 
-	NOTE : append function will have three arguments (original object,
+ NOTE
+	append function will have three arguments (original object,
 	value to append, misc data) and will return the resulting object
 	(see hash_append_str for an example).
-*/
+ ***********************************************************************/
 
 htable *hash_init_adv(size_t table_size, void *(*dupfunc)(void *),
-		void (*freefunc)(void *),
-		void *(*appdfunc)(void *, void *, void *)) {
+							void (*freefunc)(void *),
+							void *(*appdfunc)(void *, void *, void *)) {
 
-	unsigned int	 i;
-	htable		*pht;
-	hnode		*phn;
+	htable	*pht;
+	hnode	*phn;
+	size_t	 i;
 
 	/* allocate hash table */
 	pht = (htable *) malloc(sizeof(htable));
@@ -144,7 +164,7 @@ htable *hash_init_adv(size_t table_size, void *(*dupfunc)(void *),
 	/* finish init */
 	pht->size = table_size;
 	pht->count = 0;
-	pht->autogrow = 0; /* disabled by default */
+	pht->autogrow = false; /* disabled by default */
 	pht->dupobj = dupfunc;
 	pht->freeobj = freefunc;
 	pht->appdobj = appdfunc;
@@ -153,22 +173,27 @@ htable *hash_init_adv(size_t table_size, void *(*dupfunc)(void *),
 	return(pht);
 }
 
-/*
+/*****************
+ * hash_resize() *
+ ***********************************************************************
+ DESCR
 	resizing hash table
 
+ IN
 	ht : hash table to resize
 	newsize : new size
 
-	return : boolean
-*/
+ OUT
+	boolean
+ ***********************************************************************/
 
 bool hash_resize(htable *ht, size_t newsize) {
-	hcell		*phc,
-			*next;
-	hnode		*newhn;
-	size_t		 c = 0;
-	unsigned int	 h,
-			 i;
+	hcell			*phc,
+					*next;
+	hnode			*newhn;
+	size_t			 c = 0,
+					 i;
+	unsigned int	 h;
 
 	/* allocate new node table */
 	newhn = (hnode *) malloc(sizeof(hnode) * newsize);
@@ -200,29 +225,43 @@ bool hash_resize(htable *ht, size_t newsize) {
 	return(true);
 }
 
-/*
+
+/*******************
+ * hash_set_grow() *
+ ***********************************************************************
+ DESCR
 	set hash table as automatically resizable
 
+ IN
 	ht : hash table to set
-*/
+
+ OUT
+	-
+ ***********************************************************************/
 
 void hash_set_grow(htable *ht) {
-	ht->autogrow = 1;
+	ht->autogrow = true;
 }
 
-/*
+
+/******************
+ * hash_destroy() *
+ ***********************************************************************
+ DESCR
 	destroy hash table
 
+ IN
 	pht : hash table to remove
 
-	return : number of keys deleted
-*/
+ OUT
+	number of keys deleted
+ ***********************************************************************/
 
 size_t hash_destroy(htable *pht) {
-	hcell		*p,
-			*t;
-	size_t		 s,
-			 c = 0;
+	hcell			*p,
+					*t;
+	size_t			 s,
+					 c = 0;
 	unsigned int	 i;
 
 	if (pht == NULL)
@@ -248,45 +287,55 @@ size_t hash_destroy(htable *pht) {
 	return(c);
 }
 
-/*
+/**************
+ * hash_add() *
+ ***********************************************************************
+ DESCR
 	add a key in the hash table
 
+ IN
 	pht : hash structure
 	key : key string
 	value : value object
 
-	return : an error code
+ OUT
+	return an error code
 		HASH_ADD_FAIL : addition failed.
 		HASH_ADD_OKAY : added (no collision).
 		HASH_ADD_COLL : added (collision, key chained).
 		HASH_ADD_UPDT : key already exists, change value.
-*/
+ ***********************************************************************/
 
 unsigned int hash_add(htable *pht, char *key, void *value) {
 	return(hash_update(pht, key, value));
 }
 
 
-/*
+/*****************
+ * hash_update() *
+ ***********************************************************************
+ DESCR
 	update or add a key in the hash table
 
+ IN
 	pht : hash structure
 	key : key string
 	value : value object
 
-	return : an error code
+ OUT
+	return an error code
 		HASH_ADD_FAIL : addition failed.
 		HASH_ADD_OKAY : added (no collision).
 		HASH_ADD_COLL : added (collision, key chained).
 		HASH_ADD_UPDT : key already exists, change value.
-*/
+ ***********************************************************************/
 
 unsigned int hash_update(htable *pht, char *key, void *value) {
 	unsigned int	 hash,
-			 rval;
-	hnode		*phn;
-	hcell		*phc = NULL;
-	size_t		 size;
+					 rval;
+	hnode			*phn;
+	hcell			*phc = NULL;
+	size_t			 size;
 
 	rval = HASH_ADD_FAIL;
 
@@ -295,7 +344,7 @@ unsigned int hash_update(htable *pht, char *key, void *value) {
 	/* test remaing free cells */
 	if (pht->count >= size) {
 		/* number of cell is higher than the supposed table size */
-		if (pht->autogrow == 0) {
+		if (pht->autogrow == false) {
 			/* fixed size */
 			return(HASH_ADD_FAIL);
 		} else {
@@ -307,7 +356,7 @@ unsigned int hash_update(htable *pht, char *key, void *value) {
 		}
 	}
 
-	
+
 	phc = (hcell *) malloc(sizeof(hcell));
 	if (phc == NULL)
 		return(rval);
@@ -331,38 +380,49 @@ unsigned int hash_update(htable *pht, char *key, void *value) {
 	return(rval);
 }
 
-/*
+/*********************
+ * hash_update_dup() *
+ ***********************************************************************
+ DESCR
 	update or add a key in the hash table but duplicate the value
 
+ IN
 	pht : hash structure
 	key : key string
 	value : value object
 
-	return : an error code
+ OUT
+	return an error code
 		HASH_ADD_FAIL : addition failed.
 		HASH_ADD_OKAY : added (no collision).
 		HASH_ADD_COLL : added (collision, key chained).
 		HASH_ADD_UPDT : key already exists, change value.
-*/
+ ***********************************************************************/
 
 unsigned int hash_update_dup(htable *pht, char *key, void *value) {
 	return(hash_update(pht, key, pht->dupobj(value)));
 }
 
 
-/*
+/*******************
+ * hash_add_cell() *
+ ***********************************************************************
+ DESCR
 	add a new cell in a node
 
+ IN
 	phn : storage node
 	phc : cell to add
 
-	return : an error code
+ OUT
+	return an error code
 		HASH_ADD_OKAY : added (no collision).
 		HASH_ADD_COLL : added (collision, key chained).
 		HASH_ADD_UPDT : key already exists, change value.
 
-	NOTE : this function does not increment the cell number of the hash table.
-*/
+ NOTE
+	this function does not increment the cell number of the hash table.
+ ***********************************************************************/
 
 unsigned int hash_add_cell(hnode *phn, hcell *phc) {
 	hcell	*np;
@@ -376,7 +436,7 @@ unsigned int hash_add_cell(hnode *phn, hcell *phc) {
 		return(HASH_ADD_OKAY);
 	} else {
 		/* collision, hash code already used */
-		np = phn->first;	
+		np = phn->first;
 
 		/* looking for last element */
 		while (1) {
@@ -405,35 +465,46 @@ unsigned int hash_add_cell(hnode *phn, hcell *phc) {
 	}
 }
 
-/*
+
+/********************
+ * hash_add_array() *
+ ***********************************************************************
+ DESCR
 	add an array into the hash table
 
+ IN
 	ht : storage hash table
 	ary : array to add
 	size : size of the array
 
-	return : boolean
-*/
+ OUT
+	boolean
+ ***********************************************************************/
 
 bool hash_add_array(htable *pht, hpair *php, size_t size) {
 	return(hash_add_array_adv(pht, php, size, (void *(*)(void *)) strdup));
 }
 
-/*
+/************************
+ * hash_add_array_adv() *
+ ***********************************************************************
+ DESCR
 	add an array into the hash table
 
+ IN
 	ht : storage hash table
 	ary : array to add
 	size : size of the array
 	dup_func : object duplication function
 
-	return : boolean
-*/
+ OUT
+	boolean
+ ***********************************************************************/
 
 bool hash_add_array_adv(htable *pht, hpair *php, size_t size, void *(dupfunc)(void *)) {
-	bool		 error = false,
-			 rval = false;
-	htable		*pmht;
+	bool			 error = false,
+					 rval = false;
+	htable			*pmht;
 	unsigned int	 i;
 
 	pmht = hash_init(size);
@@ -456,22 +527,29 @@ bool hash_add_array_adv(htable *pht, hpair *php, size_t size, void *(dupfunc)(vo
 }
 
 
-
-/*
+/*****************
+ * hash_append() *
+ ***********************************************************************
+ DESCR
 	append a value to the existing value
 
+ IN
 	pht : hash structure
 	key : key to be appended
 	value : value to append
 	misc : extra data to transmit to append function
 
-	return : an error code.
-*/
+ OUT
+	return an error code.
+		HASH_ADD_OKAY : added (no collision).
+		HASH_ADD_COLL : added (collision, key chained).
+		HASH_ADD_UPDT : key already exists, change value.
+ ***********************************************************************/
 
 unsigned int hash_append(htable *pht, char *key, void *value, void *misc) {
 	unsigned int	 rval;
-	void		*pobj,
-			*robj;
+	void			*pobj,
+					*robj;
 
 	if (value == NULL) {
 #ifdef HASH_DEBUG
@@ -508,16 +586,24 @@ unsigned int hash_append(htable *pht, char *key, void *value, void *misc) {
 	return(rval);
 }
 
-/*
+
+/*****************
+ * hash_delete() *
+ ***********************************************************************
+ DESCR
 	remove a key from the hash table
 
+ IN
 	pht : hash table
 	key : key to search
-*/
+
+ OUT
+	-
+ ***********************************************************************/
 
 void hash_delete(htable *pht, char *key) {
-	hcell		*phc,
-			*last;
+	hcell			*phc,
+					*last;
 	unsigned int	 hash;
 
 	/* compute hash code */
@@ -559,17 +645,86 @@ void hash_delete(htable *pht, char *key) {
 	/* key not found */
 }
 
-/*
-	get a key from the hash table
 
+/******************
+ * hash_extract() *
+ ***********************************************************************
+ DESCR
+	remove a key from the hash table
+
+ IN
 	pht : hash table
 	key : key to search
 
-	return :  value or NULL
-*/
+ OUT
+	-
+ ***********************************************************************/
+
+void *hash_extract(htable *pht, char *key) {
+	hcell			*phc,
+					*last;
+	unsigned int	 hash;
+	void			*data = NULL;
+
+	/* compute hash code */
+	hash = hash_compute(key, pht->size);
+
+	phc = pht->nodetab[hash].first;
+	last = NULL;
+	while (phc != NULL) {
+		/* hash not empty */
+		if (strncmp(key, phc->key, MAX_HASH_KEY_LEN) == 0) {
+			/* found key */
+			if (last == NULL) {
+				/* first cell */
+				if (phc->next == NULL) {
+					/* only one cell */
+					pht->nodetab[hash].first = NULL;
+					pht->nodetab[hash].last = NULL;
+				} else {
+					/* re-link with next cell */
+					pht->nodetab[hash].first = phc->next;
+				}
+			} else {
+				last->next = phc->next;
+				if (phc->next == NULL) {
+					/* delete last cell, update node */
+					pht->nodetab[hash].last = last;
+				}
+			}
+			data = phc->value;
+			phc->value = NULL;
+			hash_free_hcell(pht, phc);
+			pht->count--;
+
+			break;
+		} else {
+			/* go to next cell */
+			last = phc;
+			phc = phc->next;
+		}
+	}
+	/* key not found */
+	return(data);
+}
+
+
+/**************
+ * hash_get() *
+ ***********************************************************************
+ DESCR
+	get a key from the hash table
+
+ IN
+	pht : hash table
+	key : key to search
+
+ OUT
+	return the value or NULL
+ ***********************************************************************/
 
 void *hash_get(htable *pht, char *key) {
-	hcell		*phc;
+	hcell			*phc;
 	unsigned int	 hash;
 
 	/* compute hash code */
@@ -594,19 +749,24 @@ void *hash_get(htable *pht, char *key) {
 	return(NULL);
 }
 
-/*
+/****************
+ * hash_merge() *
+ ***********************************************************************
+ DESCR
 	merge one hash table into another
 
+ IN
 	dst_ht : destination table
 	src_ht : table to merge
 
-	return : number of merged key
-*/
+ OUT
+	number of merged key
+ ***********************************************************************/
 
 size_t hash_merge(htable *dst_ht, htable *src_ht) {
-	hcell		*p;
-	size_t		 s,
-			 c = 0;
+	hcell			*p;
+	size_t			 s,
+					 c = 0;
 	unsigned int	 i;
 
 	/* get table size */
@@ -631,38 +791,48 @@ size_t hash_merge(htable *dst_ht, htable *src_ht) {
 	return(c);
 }
 
-/*
+/****************
+ * hash_nbkey() *
+ ***********************************************************************
+ DESCR
 	get number of keys
 
+ IN
 	pht : hash table
 
-	return : number of keys
-*/
+ OUT
+	number of keys
+ ***********************************************************************/
 
 size_t hash_nbkey(htable *pht) {
 	return(pht->count);
 }
 
-/*
+/***************
+ * hash_keys() *
+ ***********************************************************************
+ DESCR
 	get the keys of the hash table
 
+ IN
 	pht : hash table
 
-	return : an hkeys structure
+ OUT
+	hkeys structure
 
-	NOTE: don't forget to free the array after use.
-*/
+ NOTE
+	don't forget to free the array after use.
+ ***********************************************************************/
 
 hkeys *hash_keys(htable *pht) {
-	hcell		*p;
-	hkeys		*phk;
+	hcell			*p;
+	hkeys			*phk;
 	unsigned int	 i,
-			 j = 0;
+					 j = 0;
 
 	/* init hkeys struct to be returned */
 	phk = (hkeys *)malloc(sizeof(hkeys));
 	if (phk != NULL) {
-
 		if (pht->count == 0)
 			return(NULL);
 
@@ -680,29 +850,72 @@ hkeys *hash_keys(htable *pht) {
 					p = p->next;
 				}
 			}
-	
+
 			return(phk);
 		} else {
 #ifdef HASH_DEBUG
-                        debugf("keys alloc failed");
+			debugf("keys alloc failed");
 #endif
 			/* free allocated space only */
 			free(phk);
 		}
 #ifdef HASH_DEBUG
-        } else {
-                debugf("hkeys struct alloc failed");
+	} else {
+		debugf("hkeys struct alloc failed");
 #endif
 	}
 
 	return(NULL);
 }
 
-/*
+
+/**********************
+ * hash_keys_sorted() *
+ ***********************************************************************
+ DESCR
+	get the sorted list of keys of the hash table
+
+ IN
+	pht : hash table
+
+ OUT
+	return an hkeys structure
+
+ NOTE
+	don't forget to free the array after use
+ ***********************************************************************/
+
+static int hash_strcmp(const void *a, const void *b) {
+	return(strcmp(*(char * const *)a, *(char * const *) b));
+}
+
+hkeys *hash_keys_sorted(htable *pht) {
+	hkeys	*phk;
+
+	/* build hkey structure */
+	phk = hash_keys(pht);
+
+	if (phk != NULL) {
+		/* sort key names */
+		qsort((void *) phk->keys, phk->nkey, sizeof(char *),
+							hash_strcmp);
+	}
+	return(phk);
+}
+
+
+/*********************
+ * hash_free_hcell() *
+ ***********************************************************************
+ DESCR
 	free memory allocated to the hcell structure
 
+ IN
 	phc : structure to free
-*/
+
+ OUT
+	-
+ ***********************************************************************/
 
 void hash_free_hcell(htable *pht, hcell *phc) {
 	if (phc != NULL) {
@@ -720,11 +933,18 @@ void hash_free_hcell(htable *pht, hcell *phc) {
 	}
 }
 
-/*
+/*********************
+ * hash_free_hkeys() *
+ ***********************************************************************
+ DESCR
 	free memory allocated to the hkeys structure
 
+ IN
 	phk : structure to free
-*/
+
+ OUT
+	-
+ ***********************************************************************/
 
 void hash_free_hkeys(hkeys *phk) {
 	/* doesn't free pointed values */
@@ -732,9 +952,20 @@ void hash_free_hkeys(hkeys *phk) {
 	free(phk);
 }
 
-/*
+/*********************
+ * hash_str_append() *
+ ***********************************************************************
+ DESCR
 	append function for string hash
-*/
+
+ IN
+	orig : XXX
+	value : XXX
+	sep : XXX
+
+ OUT
+	-
+ ***********************************************************************/
 
 void *hash_str_append(void *orig, void *value, void *sep) {
 	char	*pbuf;
@@ -784,3 +1015,5 @@ void *hash_str_append(void *orig, void *value, void *sep) {
 	free(value);
 	return((void *) pbuf);
 }
+
+/* vim: set noexpandtab tabstop=4 softtabstop=4 shiftwidth=4: */
