@@ -1,8 +1,8 @@
-/* $Id: pathtools.c,v 1.16 2004/12/27 21:22:01 mipsator Exp $ */
+/* $Id: pathtools.c 1462 2005-08-21 08:52:06Z mipsator $ */
 
 
 /*
- * Copyright (c) 2003-2004 Damien Couderc
+ * Copyright (c) 2003-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,37 +44,50 @@
 
 #include <sys/param.h>
 #include <sys/stat.h>
-#include <stdio.h>
 #include <stdlib.h>
 
 #include "compat/pmk_stdbool.h"
+#include "compat/pmk_stdio.h"
 #include "compat/pmk_string.h"
 #include "pathtools.h"
 
-/*
+
+/*************
+ * chkpath() *
+ ***********************************************************************
+ DESCR
 	check a path and resolve "./", "../" and extra '/' characters
 
+ IN
 	path : path to check
 	buffer : MAXPATHLEN sized buffer that will contain resolved path
 
+ OUT
 	returns bool if check is well done
-*/
+ ***********************************************************************/
 
 bool chkpath(char *path, char *buffer) {
-	bool	 bexit = false,
-		 dot = false,
-		 dotdot = false,
-		 sep = false;
-	char	*pstr,
-		*pbuf;
-	int	 s = MAXPATHLEN;
+	bool	 loop = true,
+			 dot = false,
+			 dotdot = false,
+			 sep = false;
+	char	*pbuf;
+	int		 s = MAXPATHLEN;
+
+	if (s < 2) {
+		/* not enough space for at least "." */
+		/* XXX printf("not enough space\n"); */
+		return(false);
+	}
 
-	pstr = path;
 	pbuf = buffer;
 
-	while ((bexit == false) && (s > 0)) {
-		switch (*pstr) {
+	while ((loop == true) && (s > 1)) {
+		switch (*path) {
 			case CHAR_DOT :
+				/* unset separator flag */
+				sep = false;
+
 				/* found a dot */
 				if (dot == false) {
 					dot = true;
@@ -82,118 +95,165 @@ bool chkpath(char *path, char *buffer) {
 					if (dotdot == false) {
 						dotdot = true;
 					} else {
-						/* third dot found ??? */
+						/* third dot found !!! */
+						/* printf("triple dot found\n"); */
 						return(false);
 					}
 				}
 				break;
 
 			case CHAR_EOS :
-				bexit = true;
-				/* process end of line as a separator too */
+				/* exit from loop */
+				loop = false;
+
+				/* but consider end of string as a separator before */
+				/* no break */
 
 			case CHAR_SEP :
 				/* found separator */
 				if (dot == true) {
+					/*
+						if previous directory character sequence is not
+						found then current directory character sequence
+						is ignored
+					*/
 					if (dotdot == true) {
-						/* found "../", going one dir back if possible */
-							
-						pbuf--; /* go back on last char */
-						s++;
+						/* if not at the begining of the buffer */
+						if (pbuf != buffer) {
+							/* if not root look for previous directory */
+							if ((pbuf - buffer) > 1) {
+								/* skip separator */
+								pbuf = pbuf - 2;
+								s = s + 2;
 
-						if (pbuf > buffer) {
-							/* skip trailing '/' if not root */
-							pbuf--;
-							s++;
+								/* look for separator */
+								while ((*pbuf != CHAR_SEP) && (pbuf > buffer)) {
+									pbuf--;
+									s++;
+								}
+
+								/* if separator found go next char */
+								if (*pbuf == CHAR_SEP) {
+									pbuf++;
+									s--;
+								}
+							}
+						} else {
+							/* try to put dotdotsep */
+							if (s > 3) {
+
+								*pbuf = CHAR_DOT;
+								pbuf++;
+								*pbuf = CHAR_DOT;
+								pbuf++;
+								*pbuf = CHAR_SEP;
+								pbuf++;
+								s = s - 3;
+							} else {
+								/* not enough space in buffer */
+								/* XXX printf("not enough space for dotdotsep\n"); */
+								return(false);
+							}
 						}
 
-						/* now searching previous dir */
-						while ((*pbuf != CHAR_SEP) && (pbuf > buffer)) {
-							pbuf--;
-							s++;
-						}
-
-						/* okay found, now set pointer to next char */
-						pbuf++;
-						s--;
-
+						/* unset dotdot flag */
 						dotdot = false;
 					}
-					/* if found only "./" then just ignore */
+
+					/* unset dot flag */
 					dot = false;
 				} else {
 					if (sep == false) {
 						/* previous was not a separator so copy it */
-						sep = true;
-						*pbuf = *pstr;
+						*pbuf = *path;
 						pbuf++;
 						s--;
+
+						/* set flag */
+						sep = true;
 					}
-					/* if previous har was a separator the drop it */
 				}
 				break;
 
 			default :
-				/* dot already found ? */
 				if (dot == true) {
-					if (dotdot == true) {
-						/* should have a separator after two dot ??? */
-						return(false);
-					}
+					/* copy char in buffer */
+					*pbuf = CHAR_DOT;
+					pbuf++;
+					s--; /* XXX check */
 
-					/* hidden dir */
-					if (s > 1) {
-						/* got enough space for dot and current char */
+					if (dotdot == true) {
+						/* copy char in buffer */
 						*pbuf = CHAR_DOT;
 						pbuf++;
-						s--;
-					} else {
-						/* not enough space */
-						return(false);
+						s--; /* XXX check */
 					}
-					dot = false;
 				}
 
-				/* copy char */
-				*pbuf = *pstr;
+				/* copy char in buffer */
+				*pbuf = *path;
 				pbuf++;
 				s--;
 
-				if (sep == true) {
-					sep = false;
-				}
-				break;
+				/* unset flags */
+				sep = false;
+				dot = false;
+				dotdot = false; /* XXX let this happen ? ("..string" for example) */
 		}
-		pstr++;
+
+		/* next character */
+		path++;
 	}
 
-	if (sep == true) {
-		/*  skip trailing separator */
-		pbuf--;
-		s++;
-	}
-
-	if (s > 0) {
-		/* NULL terminate the string */
-		*pbuf = CHAR_EOS;
-		return(true);
-	} else {
+	/* if end of string has not been reached */
+	if (loop == true) {
+		/* XXX printf("s = %d, loop == true\n", s);*/
 		return(false);
 	}
+
+	switch(pbuf - buffer) {
+		case 0 :
+			*pbuf = CHAR_DOT;
+			pbuf++;
+			break;
+
+		case 1 :
+			/* don't try to skip trailing separator */
+			break;
+
+		default :
+			if (*(pbuf - 1) == CHAR_SEP) {
+				/* skip trailing character */
+				pbuf--;
+			}
+	}
+
+	/* NULL terminate string */
+	*pbuf = CHAR_EOS;
+
+	return(true);
 }
 
-/*
+
+/*************
+ * relpath() *
+ ***********************************************************************
+ DESCR
 	provide a relative path
 
+ IN
 	from : source path
 	to : destination path
 	buffer : MAXPATHLEN sized buffer that will contain relative path
-*/
+
+ OUT
+	boolean
+ ***********************************************************************/
 
 bool relpath(char *from, char *to, char *buffer) {
 	bool	bexit = false;
 	char	from_buf[MAXPATHLEN],
-		to_buf[MAXPATHLEN];
+			to_buf[MAXPATHLEN];
 
 	/* check 'from' path */
 	if (chkpath(from, from_buf) != true) {
@@ -253,15 +313,21 @@ bool relpath(char *from, char *to, char *buffer) {
 	return(true);
 }
 
-/*
+
+/*************
+ * abspath() *
+ ***********************************************************************
+ DESCR
 	create a "valid" absolute path
 
+ IN
 	base : absolute base path
 	rel : relative path from the base
 	buffer : MAXPATHLEN buffer that will contain resulting absolute path
 
+ OUT
 	returns true on success
-*/
+ ***********************************************************************/
 
 bool abspath(char *base, char *rel, char *buffer) {
 	char	tmp_buf[MAXPATHLEN]; /* XXX should be greater ? */
@@ -279,16 +345,25 @@ bool abspath(char *base, char *rel, char *buffer) {
 	return(chkpath(tmp_buf, buffer));
 }
 
-/*
+
+/**************
+ * uabspath() *
+ ***********************************************************************
+ DESCR
 	provide an absolute path from a path with unknown type.
 
+ IN
 	base : absolute base path
 	upath : path with unknown type (relative or absolute)
 	buffer : MAXPATHLEN sized buffer
 
-	NOTE : if upath is absolute then buffer is filled with
+ OUT
+	boolean
+
+ NOTE
+	if upath is absolute then buffer is filled with
 	upath else the buffer will contain the computed path.
-*/
+ ***********************************************************************/
 
 bool uabspath(char *base, char *upath, char *buffer) {
 	if (*upath == CHAR_SEP) {
@@ -300,20 +375,26 @@ bool uabspath(char *base, char *upath, char *buffer) {
 	}
 }
 
-/*
+
+/**************
+ * makepath() *
+ ***********************************************************************
+ DESCR
 	build path if it does not exists
 
+ IN
 	path : path to build
 	mode : permissions used to create new directories
 
+ OUT
 	return : boolean
-*/
+ ***********************************************************************/
 
 bool makepath(char *path, mode_t mode) {
 	bool		 bexit = false;
 	char		 save,
-			*pstr,
-			*copy;
+				*pstr,
+				*copy;
 	struct stat	 sb;
 
 	if (*path != CHAR_SEP) {
@@ -355,3 +436,4 @@ bool makepath(char *path, mode_t mode) {
 	free(copy);
 	return(true);
 }
+
