@@ -1,12 +1,7 @@
-/* $Id: pmk.c,v 1.137 2005/01/24 14:21:50 mipsator Exp $ */
+/* $Id: pmk.c 1941 2006-12-08 17:08:28Z coudercd $ */
 
 /*
- * Credits for patches :
- *	- Pierre-Yves Ritschard
- */
-
-/*
- * Copyright (c) 2003-2004 Damien Couderc
+ * Copyright (c) 2003-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,130 +33,64 @@
  *
  */
 
+/*
+ * Credits for patches :
+ *	- Pierre-Yves Ritschard
+ */
+
 
 #include <sys/stat.h>
 /* include it first as if it was <sys/types.h> - this will avoid errors */
 #include "compat/pmk_sys_types.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <stdlib.h>
 
 #include "compat/pmk_ctype.h"
 #include "compat/pmk_libgen.h" /* basename, dirname */
+#include "compat/pmk_stdio.h"
 #include "compat/pmk_string.h" /* strlcpy */
 #include "compat/pmk_unistd.h"
 #include "autoconf.h"
 #include "common.h"
 #include "func.h"
+#include "hash_tools.h"
+#include "lang.h"
 #include "pathtools.h"
 #include "pmk.h"
+#include "tags.h"
 
 /*#define PMK_DEBUG	1*/
 
-extern char	*optarg;
-extern int	 optind;
+
+/*************
+ * variables *
+ ***********************************************************************/
+
+extern char		*optarg;
+extern int		 optind;
 extern prskw	 kw_pmkfile[];
 extern size_t	 nbkwpf;
 
-int		 cur_line = 0;
+int				 cur_line = 0;
 
 
-/*
-	init variables
+/******************
+ * init functions *
+ ***********************************************************************/
 
-	pht : global data structure
-	template : file to process
+/******************
+ * pmkdata_init() *
+ ***********************************************************************
+ DESCR
+	init pmkdata structure
 
-	return : -
-*/
+ IN
+	NONE
 
-bool process_dyn_var(pmkdata *pgd, char *template) {
-	char	*srcdir,
-		*basedir,
-		*pstr,
-		 buf[MAXPATHLEN],
-		 rpath[MAXPATHLEN],
-		 stpath[MAXPATHLEN],
-		 btpath[MAXPATHLEN];
-	htable	*pht;
-
-	pht = pgd->htab;
-
-	srcdir = pgd->srcdir;
-	basedir = pgd->basedir;
-
-	/* get template path */
-	/* NOTE : we use strdup to avoid problem with linux's dirname */
-	pstr = strdup(template);
-	if (pstr == NULL) {
-		free(pstr);
-		errorf(ERRMSG_MEM);
-		return(false);
-	}
-
-	if (strlcpy_b(stpath, dirname(pstr), sizeof(stpath)) == false) {
-		free(pstr);
-		errorf(PMK_ERR_OVRFLOW);
-		return(false);
-	}
-	free(pstr);
-
-	/* compute relative path */
-	relpath(srcdir, stpath, rpath);
-
-	/* compute builddir_abs with relative path */
-	abspath(basedir, rpath, btpath);
-	if (hash_update_dup(pht, PMK_DIR_BLD_ABS, btpath) == HASH_ADD_FAIL)
-		return(false);
-#ifdef PMK_DEBUG
-	debugf("%s = '%s'", PMK_DIR_BLD_ABS, buf);
-#endif
-
-	/* compute relative path to builddir root */
-	relpath(btpath, basedir, buf);
-	if (hash_update_dup(pht, PMK_DIR_BLD_ROOT_REL, buf) == HASH_ADD_FAIL)
-		return(false);
-#ifdef PMK_DEBUG
-	debugf("%s = '%s'", PMK_DIR_BLD_ROOT_REL, buf);
-#endif
-
-	/* set buildir_rel to '.', useful ? */
-	if (hash_update_dup(pht, PMK_DIR_BLD_REL, ".") == HASH_ADD_FAIL)
-		return(false);
-#ifdef PMK_DEBUG
-	debugf("%s = '%s'", PMK_DIR_BLD_REL, buf);
-#endif
-
-	/* compute and set relative path from basedir to srcdir */
-	relpath(btpath, srcdir, buf);
-	if (hash_update_dup(pht, PMK_DIR_SRC_ROOT_REL, buf) == HASH_ADD_FAIL)
-		return(false);
-#ifdef PMK_DEBUG
-	debugf("%s = '%s'", PMK_DIR_SRC_ROOT_REL, buf);
-#endif
-
-	/* set absolute path of template */
-	if (hash_update_dup(pht, PMK_DIR_SRC_ABS, stpath) == HASH_ADD_FAIL)
-		return(false);
-#ifdef PMK_DEBUG
-	debugf("%s = '%s'", PMK_DIR_SRC_ABS, buf);
-#endif
-
-	/* compute and set relative path from template to builddir */
-	relpath(btpath, stpath, buf);
-	if (hash_update_dup(pht, PMK_DIR_SRC_REL, buf) == HASH_ADD_FAIL)
-		return(false);
-#ifdef PMK_DEBUG
-	debugf("%s = '%s'", PMK_DIR_SRC_REL, buf);
-#endif
-
-	return(true);
-}
-
-/*
-	init variables
-*/
+ OUT
+	pmkdata structure
+ ***********************************************************************/
 
 pmkdata *pmkdata_init(void) {
 	pmkdata	*ppd;
@@ -184,12 +113,22 @@ pmkdata *pmkdata_init(void) {
 		return(NULL);
 	}
 
-	/* init template list */
-	ppd->tlist = NULL;
+	if (init_compiler_data(&(ppd->comp_data), LANG_NUMBER) == false) {
+		hash_destroy(ppd->htab);
+		hash_destroy(ppd->labl);
+		errorf("cannot initialize compiler data structure.");
+		return(NULL);
+	}
+
+	/* init compiler detection flag */
+	ppd->sys_detect = false;
 
 	/* init autoconf file */
 	ppd->ac_file = NULL;
 
+	/* init template list */
+	ppd->tlist = NULL;
+
 	/* init default language */
 	ppd->lang = NULL;
 
@@ -199,17 +138,23 @@ pmkdata *pmkdata_init(void) {
 	return(ppd);
 }
 
-/*
+
+/**************
+ * init_var() *
+ ***********************************************************************
+ DESCR
 	init variables
 
-	pgd : global data structure
+ IN
+	pgd :	global data structure
 
-        return : -
-*/
+ OUT
+	NONE
+ ***********************************************************************/
 
 bool init_var(pmkdata *pgd) {
 	char	 buf[TMP_BUF_LEN],
-                *pstr;
+			*pstr;
 	htable	*pht;
 
 	pht = pgd->htab;
@@ -221,8 +166,9 @@ bool init_var(pmkdata *pgd) {
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
 #endif
-	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
+	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL) {
 		return(false);
+	}
 
 	pstr = MK_VAR_CXXFLAGS;
 	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
@@ -231,8 +177,9 @@ debugf("%s = '%s'", pstr, buf);
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
 #endif
-	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
+	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL) {
 		return(false);
+	}
 
 	pstr = MK_VAR_CPPFLAGS;
 	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
@@ -241,8 +188,9 @@ debugf("%s = '%s'", pstr, buf);
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
 #endif
-	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
+	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL) {
 		return(false);
+	}
 
 	pstr = MK_VAR_LDFLAGS;
 	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
@@ -251,8 +199,31 @@ debugf("%s = '%s'", pstr, buf);
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
 #endif
-	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
+	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL) {
 		return(false);
+	}
+
+	pstr = MK_VAR_CLDFLAGS;
+	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
+		buf[0] = CHAR_EOS; /* empty string */
+	}
+#ifdef PMK_DEBUG
+debugf("%s = '%s'", pstr, buf);
+#endif
+	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL) {
+		return(false);
+	}
+
+	pstr = MK_VAR_CXXLDFLAGS;
+	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
+		buf[0] = CHAR_EOS; /* empty string */
+	}
+#ifdef PMK_DEBUG
+debugf("%s = '%s'", pstr, buf);
+#endif
+	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL) {
+		return(false);
+	}
 
 	pstr = MK_VAR_LIBS;
 	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
@@ -261,8 +232,9 @@ debugf("%s = '%s'", pstr, buf);
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
 #endif
-	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
+	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL) {
 		return(false);
+	}
 
 	pstr = MK_VAR_DEBUG;
 	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
@@ -271,79 +243,112 @@ debugf("%s = '%s'", pstr, buf);
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
 #endif
-	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
-		return(false);
 
-	/* autoconf shit ? */
-	if (hash_update_dup(pht, "OBJEXT", "o") == HASH_ADD_FAIL)
+	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL) {
 		return(false);
+	}
+
+	/* autoconf shit ? *//* XXX YES !!! => to move ! */
+	if (hash_update_dup(pht, "OBJEXT", "o") == HASH_ADD_FAIL) {
+		return(false);
+	}
 
 
 	pstr = hash_get(pht, PMKCONF_BIN_CC);
 	if (pstr != NULL) {
-		if (hash_update_dup(pht, "CC", pstr) == HASH_ADD_FAIL)
+		if (hash_update_dup(pht, "CC", pstr) == HASH_ADD_FAIL) {
 			return(false);
+		}
 	}
 
 	pstr = hash_get(pht, PMKCONF_BIN_CXX);
 	if (pstr != NULL) {
-		if (hash_update_dup(pht, "CXX", pstr) == HASH_ADD_FAIL)
+		if (hash_update_dup(pht, "CXX", pstr) == HASH_ADD_FAIL) {
 			return(false);
+		}
 	}
 
 	pstr = hash_get(pht, PMKCONF_BIN_CPP);
 	if (pstr != NULL) {
-		if (hash_update_dup(pht, "CPP", pstr) == HASH_ADD_FAIL)
+		if (hash_update_dup(pht, "CPP", pstr) == HASH_ADD_FAIL) {
 			return(false);
+		}
 	}
 
 	pstr = hash_get(pht, PMKCONF_BIN_INSTALL);
 	if (pstr != NULL) {
-	        /* append -c for compat with autoconf*/
-	        if (snprintf_b(buf, sizeof(buf), "%s -c", pstr) == false)
+		if (hash_update_dup(pht, "INSTALL", pstr) == HASH_ADD_FAIL) {
 			return(false);
-		if (hash_update_dup(pht, "INSTALL", buf) == HASH_ADD_FAIL)
+		}
+	}
+
+	pstr = hash_get(pht, PMKCONF_BIN_AR);
+	if (pstr != NULL) {
+		if (hash_update_dup(pht, "AR", pstr) == HASH_ADD_FAIL) {
 			return(false);
+		}
 	}
 
 	pstr = hash_get(pht, PMKCONF_BIN_RANLIB);
 	if (pstr != NULL) {
-		if (hash_update_dup(pht, "RANLIB", pstr) == HASH_ADD_FAIL)
+		if (hash_update_dup(pht, "RANLIB", pstr) == HASH_ADD_FAIL) {
 			return(false);
+		}
 	}
 
 	pstr = hash_get(pht, PMKCONF_BIN_SH);
 	if (pstr != NULL) {
-		if (hash_update_dup(pht, "SHELL", pstr) == HASH_ADD_FAIL)
+		if (hash_update_dup(pht, "SHELL", pstr) == HASH_ADD_FAIL) {
 			return(false);
+		}
 	}
 
 	pstr = hash_get(pht, PMKCONF_BIN_STRIP);
 	if (pstr != NULL) {
-		if (hash_update_dup(pht, "STRIP", pstr) == HASH_ADD_FAIL)
+		if (hash_update_dup(pht, "STRIP", pstr) == HASH_ADD_FAIL) {
 			return(false);
+		}
 	}
 
 	pstr = hash_get(pht, PMKCONF_BIN_AWK);
 	if (pstr != NULL) {
-		if (hash_update_dup(pht, "AWK", pstr) == HASH_ADD_FAIL)
+		if (hash_update_dup(pht, "AWK", pstr) == HASH_ADD_FAIL) {
 			return(false);
+		}
 	}
 
 	pstr = hash_get(pht, PMKCONF_BIN_EGREP);
 	if (pstr != NULL) {
-		if (hash_update_dup(pht, "EGREP", pstr) == HASH_ADD_FAIL)
+		if (hash_update_dup(pht, "EGREP", pstr) == HASH_ADD_FAIL) {
 			return(false);
+		}
+	}
+
+	/* shared lib support */
+	if (hash_update_dup(pht, MK_VAR_SL_BUILD, "") == HASH_ADD_FAIL) {
+		return(false);
+	}
+	if (hash_update_dup(pht, MK_VAR_SL_CLEAN, "") == HASH_ADD_FAIL) {
+		return(false);
+	}
+	if (hash_update_dup(pht, MK_VAR_SL_INST, "") == HASH_ADD_FAIL) {
+		return(false);
+	}
+	if (hash_update_dup(pht, MK_VAR_SL_DEINST, "") == HASH_ADD_FAIL) {
+		return(false);
 	}
 
 	/* set absolute paths */
-	if (hash_update_dup(pht, PMK_DIR_SRC_ROOT_ABS, pgd->srcdir) == HASH_ADD_FAIL)
-			return(false);
+	if (hash_update_dup(pht, PMK_DIR_SRC_ROOT_ABS, pgd->srcdir) == HASH_ADD_FAIL) {
+		return(false);
+	}
 #ifdef PMK_DEBUG
 	debugf("%s = '%s'", PMK_DIR_SRC_ROOT_ABS, pgd->srcdir);
 #endif
-	if (hash_update_dup(pht, PMK_DIR_BLD_ROOT_ABS, pgd->basedir) == HASH_ADD_FAIL)
-			return(false);
+
+	if (hash_update_dup(pht, PMK_DIR_BLD_ROOT_ABS, pgd->basedir) == HASH_ADD_FAIL) {
+		return(false);
+	}
 #ifdef PMK_DEBUG
 	debugf("%s = '%s'", PMK_DIR_BLD_ROOT_ABS, pgd->basedir);
 #endif
@@ -351,237 +356,423 @@ debugf("%s = '%s'", pstr, buf);
 	return(true);
 }
 
-/*
-	process the target file to replace tags
 
-	target : path of the target file
-	pgd : global data structure
+/**********************
+ * template functions *
+ ***********************************************************************/
 
-	return : boolean
-*/
+/***********************
+ * process_dyn_paths() *
+ ***********************************************************************
+ DESCR
+	process dynamic paths
 
-bool process_template(char *template, pmkdata *pgd) {
-	FILE	*tfd,
-		*dfd;
-	bool	 ac_flag;
-	char	*plb,
-		*pbf,
-		*ptn,
-		*tptn,
-		*pfn,
-		*tpfn,
-		*ptmp,
-		 lbuf[MAXPATHLEN],
-		 buf[MAXPATHLEN],
-		 tbuf[MAXPATHLEN],
-		 tpath[MAXPATHLEN],
-		 fpath[MAXPATHLEN],
-		 cibuf[TMP_BUF_LEN];
-	htable	*pht;
+ IN
+	pgd :		global data structure
+	tmplpath :	file to process
 
-	pht = pgd->htab;
+ OUT
+	NONE
+ ***********************************************************************/
 
-	/* save template name */
-	tptn = basename(template);
-	if (tptn == NULL) {
-		errorf("couldn't extract the base portion of '%s' : %s.",
-			template, strerror(errno));
-		return(false);
-	}
-	ptn = strdup(tptn);
-	if (ptn == NULL) {
-		free(ptn);
-		errorf(ERRMSG_MEM);
-		return(false);
-	}	
+bool process_dyn_paths(pmkdata *pgd, char *tmplpath) {
+	char		*srcdir,
+				*basedir,
+				 tpath[MAXPATHLEN];
+	pmkdyn_t	*pdd;
 
-	/* get the file name */
-	if (strlcpy_b(lbuf, ptn, sizeof(lbuf)) == false) {
+	pdd = &(pgd->dyndata);
+
+	srcdir = pgd->srcdir;
+	basedir = pgd->basedir;
+
+	/*
+		get template path
+
+		NOTE : we use strdup() to avoid problem with linux's dirname
+	*/
+	if (strlcpy_b(tpath, tmplpath, sizeof(tpath)) == false) {
 		errorf(PMK_ERR_OVRFLOW);
 		return(false);
 	}
 
-	/* remove the last suffix */
-	ptmp = strrchr(lbuf, '.');
-	if (ptmp != NULL) {
-		*ptmp = CHAR_EOS;
-	} else {
-		errorf("error while creating name of template.");
+	/* save template filename */
+	if (strlcpy_b(pdd->tmpl_name, basename(tpath), sizeof(pdd->tmpl_name)) == false) {
+		errorf(PMK_ERR_OVRFLOW);
 		return(false);
 	}
 
-	/* save generated file name */
-	tpfn = basename(lbuf);
-	if (tpfn == NULL) {
-		errorf("couldn't extract the base portion of '%s' : %s.",
-			lbuf, strerror(errno));
-		return(false);
-	}
-	pfn = strdup(tpfn);
-	if (pfn == NULL) {
-		free(pfn);
-		errorf(ERRMSG_MEM);
+	/* set absolute path of template */
+	if (strlcpy_b(pdd->src_abs, dirname(tpath), sizeof(pdd->src_abs)) == false) {
+		errorf(PMK_ERR_OVRFLOW);
 		return(false);
 	}
 
-	/* get relative path from srcdir */
-	/* NOTE : we use strdup to avoid problem with linux's dirname */
-	ptmp = strdup(template);
-	if (ptmp == NULL) {
-		free(ptmp);
-		errorf(ERRMSG_MEM);
+	/* compute relative path */
+	if (relpath(srcdir, pdd->src_abs, tpath) == false) {
+		errorf(PMK_ERR_OVRFLOW);
 		return(false);
 	}
-	relpath(pgd->srcdir, dirname(ptmp), buf); /* XXX check ? */
-	free(ptmp);
-	/* apply to basedir */
-	abspath(pgd->basedir, buf, tpath); /* XXX check ? */
+
+	/* compute builddir_abs with relative path */
+	if (abspath(basedir, tpath, pdd->bld_abs) == false) {
+		errorf(PMK_ERR_OVRFLOW);
+		return(false);
+	}
+
+	/* compute relative path to builddir root */
+	if (relpath(pdd->bld_abs, basedir, pdd->bld_root_rel) == false) {
+		errorf(PMK_ERR_OVRFLOW);
+		return(false);
+	}
+
+	/* set buildir_rel to '.', useful ? */
+	if (strlcpy_b(pdd->bld_rel, ".", sizeof(pdd->bld_rel)) == false) {
+		errorf(PMK_ERR_OVRFLOW);
+		return(false);
+	}
+
+	/* compute and set relative path from basedir to srcdir */
+	if (relpath(pdd->bld_abs, srcdir, pdd->src_root_rel) == false) {
+		errorf(PMK_ERR_OVRFLOW);
+		return(false);
+	}
+
+	/* compute and set relative path from template to builddir */
+	if (relpath(pdd->bld_abs, pdd->src_abs, pdd->src_rel) == false) {
+		errorf(PMK_ERR_OVRFLOW);
+		return(false);
+	}
+
+	return(true);
+}
+
+
+/*********************
+ * process_dyn_var() *
+ ***********************************************************************
+ DESCR
+	set dynamic path variables
+
+ IN
+	pgd : 		global data structure
+	template :	file to process
+
+ OUT
+	NONE
+ ***********************************************************************/
+
+bool process_dyn_var(pmkdata *pgd) {
+	htable		*pht;
+	pmkdyn_t	*pdd;
+
+	pht = pgd->htab;
+	pdd = &(pgd->dyndata);
+
+	if (hash_update_dup(pht, PMK_DIR_BLD_ROOT_REL, pdd->bld_root_rel) == HASH_ADD_FAIL) {
+		return(false);
+	}
+#ifdef PMK_DEBUG
+	debugf("%s = '%s'", PMK_DIR_BLD_ROOT_REL, pdd->bld_root_rel);
+#endif
+
+	if (hash_update_dup(pht, PMK_DIR_BLD_ABS, pdd->bld_abs) == HASH_ADD_FAIL) {
+		return(false);
+	}
+#ifdef PMK_DEBUG
+	debugf("%s = '%s'", PMK_DIR_BLD_ABS, pdd->bld_abs);
+#endif
+
+	if (hash_update_dup(pht, PMK_DIR_BLD_REL, pdd->bld_rel) == HASH_ADD_FAIL) {
+		return(false);
+	}
+#ifdef PMK_DEBUG
+	debugf("%s = '%s'", PMK_DIR_BLD_REL, pdd->bld_rel);
+#endif
+
+	if (hash_update_dup(pht, PMK_DIR_SRC_ROOT_REL, pdd->src_root_rel) == HASH_ADD_FAIL) {
+		return(false);
+	}
+#ifdef PMK_DEBUG
+	debugf("%s = '%s'", PMK_DIR_SRC_ROOT_REL, pdd->src_root_rel);
+#endif
+
+	if (hash_update_dup(pht, PMK_DIR_SRC_ABS, pdd->src_abs) == HASH_ADD_FAIL) {
+		return(false);
+	}
+#ifdef PMK_DEBUG
+	debugf("%s = '%s'", PMK_DIR_SRC_ABS, pdd->src_abs);
+#endif
+
+	if (hash_update_dup(pht, PMK_DIR_SRC_REL, pdd->src_rel) == HASH_ADD_FAIL) {
+		return(false);
+	}
+#ifdef PMK_DEBUG
+	debugf("%s = '%s'", PMK_DIR_SRC_REL, pdd->src_rel);
+#endif
+
+	return(true);
+}
+
+
+/**********************
+ * process_template() *
+ ***********************************************************************
+ DESCR
+	process the target file to replace tags
+
+ IN
+	target :	path of the target file
+	pgd :		global data structure
+
+ OUT
+	boolean
+ ***********************************************************************/
+
+bool process_template(pmkdata *pgd, char *template) {
+	FILE		*tfp,
+				*gfp;
+	bool		 ac_flag;
+	char		*gfname,
+				*pstr,
+				 gfpath[MAXPATHLEN],
+				 buf[TMP_BUF_LEN];
+	htable		*pht;
+	pmkdyn_t	*pdd;
+	prseng_t	*ppe;
+
+	pht = pgd->htab;
+	pdd = &(pgd->dyndata);
 
 	/* create path if it does not exists */
-	if (makepath(tpath, S_IRWXU | S_IRWXG | S_IRWXO) == false) {
-		errorf("cannot build template path '%s'.", tpath);
-		return(false);
-	}
-	
-	/* append filename */
-	abspath(tpath, lbuf, fpath);
-
-	tfd = fopen(template, "r");
-	if (tfd == NULL) {
-		errorf("cannot open '%s' : %s.",
-			template, strerror(errno));
+	if (makepath(pdd->bld_abs, S_IRWXU | S_IRWXG | S_IRWXO) == false) {
+		errorf("cannot build template generated file path '%s'.", pdd->bld_abs);
 		return(false);
 	}
 
-	dfd = fopen(fpath, "w");
-	if (dfd == NULL) {
-		fclose(tfd); /* close already opened tfd before leaving */
-		errorf("cannot open '%s' : %s.",
-			fpath, strerror(errno));
+	/* generate full path for generated file */
+	gfname = gen_from_tmpl(pdd->tmpl_name);
+	if (abspath(pdd->bld_abs, gfname, gfpath) == false) {
+		errorf("failed to build absolute path from '%s' and '%s'", pdd->bld_abs, gfname);
 		return(false);
 	}
 
-	/* XXX should use directly path ? */
-	if (process_dyn_var(pgd, template) == false)
-		return(false); /* XXX error message ? */
+	tfp = fopen(template, "r");
+	if (tfp == NULL) {
+		errorf("cannot open '%s' : %s.", template, strerror(errno));
+		return(false);
+	}
 
+	gfp = fopen(gfpath, "w");
+	if (gfp == NULL) {
+		fclose(tfp); /* close already opened tfp before leaving */
+		errorf("cannot open '%s' : %s.", gfpath, strerror(errno));
+		return(false);
+	}
+
+	if (process_dyn_var(pgd) == false) {
+		errorf("failed to process dynamic variables.");
+		return(false);
+	}
+
+	/* check if autoconf dynamic variables are needed */
 	if (pgd->ac_file == NULL) {
 		ac_flag = false;
 	} else {
 		ac_flag = true;
 		/* XXX should use directly path ? */
-		if (ac_process_dyn_var(pgd, template) == false)
-			return(false); /* XXX error message ? */
+		if (ac_process_dyn_var(pgd, template) == false) {
+			errorf("failed to process autoconf dynamic variables.");
+			return(false);
+		}
 	}
 
-	if (snprintf_b(cibuf, sizeof(cibuf),
-				"%s generated from %s by PMK.",
-				pfn, ptn) == false)
+	/* generate @configure_input@ tag */
+	/* XXX handle in pmkscan templates ? */
+	if (snprintf_b(buf, sizeof(buf), PMK_GENMSG, gfname, pdd->tmpl_name) == false) {
 		return(false);
-
-	if (hash_update_dup(pht, "configure_input", cibuf) == HASH_ADD_FAIL)
+	}
+	if (hash_update_dup(pht, "configure_input", buf) == HASH_ADD_FAIL) {
 		return(false);
+	}
 
-	free(ptn);
-	free(pfn);
+	/* init prseng with template */
+	ppe = prseng_init(tfp, NULL);
+	if (ppe == NULL) {
+		errorf("parse engine init failed.");
+		return(false);
+	}
 
-	while (fgets(lbuf, sizeof(lbuf), tfd) != NULL) {
-		plb = lbuf;
-		pbf = buf;
+	/* while end of template is not reached */
+	while (prseng_eof(ppe) == false) {
+		if (prseng_test_char(ppe, PMK_TAG_CHAR) == true) {
+			/* skip tag character */
+			if (prseng_next_char(ppe) == false) {
+				return(false);
+			}
 
-		while (*plb != CHAR_EOS) {
-			if (*plb == PMK_TAG_CHAR) {
-				plb++;
-				/* get tag identifier */
-				ptmp = parse_identifier(plb, tbuf, sizeof(tbuf));
-				if (ptmp == NULL) {
-					errorf("process_template(), buffer too small.", fpath);
-					return(false);
-				}
-
-				plb = ptmp;
-
-				if (*ptmp == PMK_TAG_CHAR) {
-					ptmp = (char *) hash_get(pht, tbuf);
-					if (ptmp == NULL) {
-						/* not a valid tag, put it back */
-						*pbf = PMK_TAG_CHAR; /* first tag character */
-						pbf++;
-						ptmp = tbuf;
-					} else {
-						/* tag ok, skip ending tag char */
-						plb++;
+			/* get tag identifier */
+			if (prseng_get_idtf(ppe, buf, sizeof(buf), PMK_TAG_IDTF_STR) == true) {
+				/* if valid tag identifier */
+				if (prseng_test_char(ppe, PMK_TAG_CHAR) == true) {
+					/* skip end of tag character */
+					if (prseng_next_char(ppe) == false) {
+						return(false);
 					}
 
+					/* try to get tag content */
+					pstr = (char *) hash_get(pht, buf);
+					if (pstr != NULL) {
+						/* put data */
+						fprintf(gfp, "%s", pstr);
+					} else {
+						/* else no data, put back tag def */
+						fprintf(gfp, "%c%s%c", PMK_TAG_CHAR, buf, PMK_TAG_CHAR);
+					}
 				} else {
-					/* not an identifier */
-					ptmp = tbuf;
-					*pbf = PMK_TAG_CHAR; /* first tag character */
-					pbf++;
-				}
+					/* else not a valid tag identifier */
+					fprintf(gfp, "%c%s%c", PMK_TAG_CHAR, buf, prseng_get_char(ppe));
 
-				/* copy */
-				while (*ptmp != CHAR_EOS) {
-					*pbf = *ptmp;
-					pbf++;
-					ptmp++;
+					/* skip character */
+					if (prseng_next_char(ppe) == false) {
+						return(false);
+					}
 				}
 			} else {
-				/* copy normal char */
-				*pbf = *plb;
-				pbf++;
-				plb++;
-                        }
-		}
+				/* else failed to get a tag identifier */
+				fprintf(gfp, "%c%s", PMK_TAG_CHAR, buf);
+			}
+		} else {
+			/* put character in generated file */
+			fputc(prseng_get_char(ppe), gfp);
 
-                *pbf = CHAR_EOS;
-		/* saving parsed line */
-		fprintf(dfd, "%s", buf);
+			/* next character */
+			if (prseng_next_char(ppe) == false) {
+				return(false);
+			}
+		}
 	}
 
-	fclose(dfd);
-	fclose(tfd);
+	fclose(gfp);
+	fclose(tfp);
 
-	pmk_log("Created '%s'.\n", fpath);
+	pmk_log("Created '%s'.\n", gfpath);
 
 	if (ac_flag == true) {
-		/* clean dyn_var */
+		/* clean autoconf dynamic variables */
 		ac_clean_dyn_var(pht);
 	}
 
 	hash_delete(pht, "configure_input");
-		
+
 	return(true);
 }
 
-/*
+
+/*******************
+ * template_main() *
+ ***********************************************************************
+ DESCR
+	main procedure for template management
+
+ IN
+	pgd :		global data structure
+
+ OUT
+	boolean
+ ***********************************************************************/
+
+bool template_main(pmkdata *pgd) {
+	char			*pstr,
+					 buf[MAXPATHLEN];
+	dynary			*da;
+	unsigned int	 i;
+
+	da = pgd->tlist;
+	if (da == NULL) {
+		errorf("no target given.");
+		exit(EXIT_FAILURE);
+	}
+
+	/* process each template */
+	for (i = 0 ; i < da_usize(da) ; i++) {
+		pstr = da_idx(da, i);
+		if (pstr != NULL) {
+			if (abspath(pgd->srcdir, pstr, buf) == false) {
+				errorf("failed to build absolute path of '%s'.", pstr);
+				return(false);
+			}
+
+			if (process_dyn_paths(pgd, buf) == false) {
+				errorf("failed to process dynamic paths for '%s'.", buf);
+				return(false);
+			}
+
+			if (process_template(pgd, buf) == false) {
+				/* failure while processing template */
+				errorf("failed to process template '%s'.", buf);
+				return(false);
+			}
+		}
+	}
+
+	/* process eventual autoconf style config file */
+	if (pgd->ac_file != NULL) {
+		pmk_log("\nProcess '%s' for autoconf compatibility.\n", pgd->ac_file);
+		if (ac_parse_config(pgd) == false) {
+			/* failure while processing autoconf file */
+			errorf("failed to process autoconf config file '%s'.", pgd->ac_file);
+			return(false);
+		}
+	}
+
+	return(true);
+}
+
+
+/***************************
+ * miscellaneous functions *
+ ***********************************************************************/
+
+/*****************
+ * process_cmd() *
+ ***********************************************************************
+ DESCR
 	process the parsed command
 
+ IN
 	pdata : parsed data
 	pgd : global data structure
 
-	return : boolean
-*/
+ OUT
+	boolean
+ ***********************************************************************/
 
 bool process_cmd(prsdata *pdata, pmkdata *pgd) {
 	return(process_node(pdata->tree, pgd));
 }
 
-/*
+
+/*******************
+ * parse_cmdline() *
+ ***********************************************************************
+ DESCR
 	process command line values
 
+ IN
 	val : array of defines
 	nbval : size of the array
 	pgd : global data structure
 
-	return : boolean (true on success)
-*/
+ OUT
+	boolean (true on success)
+ ***********************************************************************/
 
 bool parse_cmdline(char **val, int nbval, pmkdata *pgd) {
 	bool	 rval = true;
 	char	*pstr;
 	htable	*ht;
-	int	 i;
+	int		 i;
 	prsopt	 opt;
 
 	/* don't init pscell */
@@ -608,13 +799,19 @@ bool parse_cmdline(char **val, int nbval, pmkdata *pgd) {
 	return(rval);
 }
 
-/*
+
+/***********
+ * clean() *
+ ***********************************************************************
+ DESCR
 	clean global data
 
+ IN
 	pgd : global data structure
 
-	return : -
-*/
+ OUT
+	NONE
+ ***********************************************************************/
 
 void clean(pmkdata *pgd) {
 	if (pgd->htab != NULL) {
@@ -624,9 +821,13 @@ void clean(pmkdata *pgd) {
 	if (pgd->labl != NULL) {
 		hash_destroy(pgd->labl);
 	}
+
+	clean_compiler_data(&(pgd->comp_data));
+
 	if (pgd->cfgt != NULL) {
 		cfgtdata_destroy(pgd->cfgt);
 	}
+
 	if (pgd->tlist != NULL) {
 		da_destroy(pgd->tlist);
 	}
@@ -640,39 +841,94 @@ void clean(pmkdata *pgd) {
 	}
 }
 
-/*
+
+/*********************
+ * set_switch_list() *
+ ***********************************************************************
+ DESCR
+	set or unset a switch
+
+ IN
+	swlst :	list of switches to process
+	state :	switch state to set (true or false)
+
+ OUT
+	boolean
+ ***********************************************************************/
+
+bool set_switch_list(pmkdata *pgd, char *swlst, bool state, int *ovrsw) {
+	char	*pstr,
+			*ststr;
+	dynary	*da;
+
+	if (state == true) {
+		ststr = BOOL_STRING_TRUE;
+	} else {
+		ststr = BOOL_STRING_FALSE;
+	}
+
+	da = str_to_dynary(swlst, CHAR_LIST_SEPARATOR);
+	if (da != NULL) {
+		do {
+			pstr = da_pop(da);
+			if (pstr != NULL) {
+				if (hash_update_dup(pgd->labl, pstr, ststr) == HASH_ADD_FAIL)
+					return(false);
+				*ovrsw++; /* increment number of overriden switches */
+				free(pstr);
+			}
+		} while (pstr != NULL);
+		da_destroy(da);
+	}
+
+	return(true);
+}
+
+
+/***********
+ * usage() *
+ ***********************************************************************
+ DESCR
 	usage
-*/
+
+ IN
+	NONE
+
+ OUT
+	NONE
+ ***********************************************************************/
 
 void usage(void) {
 	fprintf(stderr, "usage: pmk [-vh] [-e list] [-d list] [-b path]\n");
 	fprintf(stderr, "\t[-f pmkfile] [-o ovrfile] [options]\n");
 }
 
-/*
-	main
-*/
+
+/**********
+ * main() *
+ ***********************************************************************
+ DESCR
+	main loop
+ ***********************************************************************/
 
 int main(int argc, char *argv[]) {
-	FILE		*fp;
-	bool		 go_exit = false,
-			 pmkfile_set = false,
-			 ovrfile_set = false,
-			 basedir_set = false,
-			 buildlog = false;
-	char		*pstr,
-			*enable_sw = NULL,
-			*disable_sw = NULL,
-			 buf[MAXPATHLEN];
-	dynary		*da;
-	int		 rval = 0,
-			 nbpd,
-			 nbcd,
-			 ovrsw = 0,
-			 chr;
-	pmkdata		*pgd;
-	prsdata		*pdata;
-	unsigned int	 i;
+	FILE			*fp;
+	bool			 go_exit = false,
+					 pmkfile_set = false,
+					 ovrfile_set = false,
+					 basedir_set = false,
+					 buildlog = false;
+	char			*pstr,
+					*enable_sw = NULL,
+					*disable_sw = NULL,
+					 buf[MAXPATHLEN];
+	int				 rval = 0,
+					 nbpd,
+					 nbcd,
+					 ovrsw = 0,
+					 chr;
+	pmkdata			*pgd;
+	prsdata			*pdata;
 
 	/* get current path */
 	if (getcwd(buf, sizeof(buf)) == NULL) {
@@ -722,7 +978,7 @@ int main(int argc, char *argv[]) {
 
 				case 'f' :
 					/* pmk file path */
-					/* XXX check if path is valid */
+					/* XXX check if path in optarg is valid */
 					if (uabspath(buf, optarg, pgd->pmkfile) == false) {
 						errorf("cannot use file argument.");
 						exit(EXIT_FAILURE);
@@ -758,7 +1014,7 @@ int main(int argc, char *argv[]) {
 
 				case 'o' :
 					/* file path to override pmk.conf */
-					/* XXX check if path is valid */
+					/* XXX check if path in optarg is valid */
 					if (uabspath(buf, optarg, pgd->ovrfile) == false) {
 						errorf("cannot use file argument.");
 						exit(EXIT_FAILURE);
@@ -807,12 +1063,17 @@ int main(int argc, char *argv[]) {
 	}
 
 	if (buildlog == true) {
+		/* set build log name */
 		if (strlcpy_b(pgd->buildlog, PMK_BUILD_LOG,
 					sizeof(pgd->buildlog)) == false) {
 			errorf(PMK_ERR_BLDLOG);
 			exit(EXIT_FAILURE);
 		}
+
+		/* remove previous build log */
+		unlink(pgd->buildlog);
 	} else {
+		/* redirect build output to /dev/null */
 		if (strlcpy_b(pgd->buildlog, "/dev/null",
 					sizeof(pgd->buildlog)) == false) {
 			errorf(PMK_ERR_BLDLOG);
@@ -855,34 +1116,17 @@ int main(int argc, char *argv[]) {
 
 	if (enable_sw != NULL) {
 		/* command line switches to enable */
-		da = str_to_dynary(enable_sw, CHAR_LIST_SEPARATOR);
-		if (da != NULL) {
-			pstr = da_pop(da);	
-			while (pstr != NULL) {
-				if (hash_update_dup(pgd->labl, pstr, BOOL_STRING_TRUE) == HASH_ADD_FAIL)
-					return(false);
-				ovrsw++; /* increment number of overriden switches */
-				free(pstr);
-				pstr = da_pop(da);	
-			}
-			da_destroy(da);
+		if (set_switch_list(pgd, enable_sw, true, &ovrsw) == false) {
+			errorf("failed to set enabled switches.");
+			exit(EXIT_FAILURE);
 		}
 	}
 
 	if (disable_sw != NULL) {
 		/* command line switches to disable */
-		da = str_to_dynary(disable_sw, CHAR_LIST_SEPARATOR);
-		if (da != NULL) {
-			do {
-				pstr = da_pop(da);
-				if (pstr != NULL) {
-					if (hash_update_dup(pgd->labl, pstr, BOOL_STRING_FALSE) == HASH_ADD_FAIL)
-						return(false);
-					ovrsw++; /* increment number of overriden switches */
-					free(pstr);
-				}
-			} while (pstr != NULL);
-			da_destroy(da);
+		if (set_switch_list(pgd, disable_sw, false, &ovrsw) == false) {
+			errorf("failed to set disabled switches.");
+			exit(EXIT_FAILURE);
 		}
 	}
 
@@ -945,63 +1189,32 @@ int main(int argc, char *argv[]) {
 	fp = fopen(pgd->pmkfile, "r");
 	if (fp == NULL) {
 		clean(pgd);
-		errorf("cannot open '%s' : %s.",
-			pgd->pmkfile, strerror(errno));
+		errorf("failed to open '%s' : %s.", pgd->pmkfile, strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
-	if (parse_pmkfile(fp, pdata, kw_pmkfile, nbkwpf) == false) {
-		/* XXX too much things here */
-		clean(pgd);
-		fflush(fp);
-		fclose(fp);
-		pmk_log_close();
-		errorf("while opening '%s'.", pgd->pmkfile);
-		exit(EXIT_FAILURE);
-	}
+	if (parse_pmkfile(fp, pdata, kw_pmkfile, nbkwpf) == true) {
+		pmk_log("\nProcessing commands :\n");
+		if (process_cmd(pdata, pgd) == false) {
+			/* an error occured while parsing */
+			rval = 1;
+		} else {
+			pmk_log("\nProcess templates :\n");
 
-	fflush(fp);
-	fclose(fp);
-
-	pmk_log("\nProcessing commands :\n");
-	if (process_cmd(pdata, pgd) == false) {
-		/* an error occured while parsing */
-		rval = 1;
-	} else {
-		pmk_log("\nProcess templates :\n");
-
-		da = pgd->tlist;
-
-		if (da == NULL) {
-			errorf("no target given.");
-			exit(EXIT_FAILURE);
-		}
-		for (i = 0 ; (i < da_usize(da)) && (rval == 0) ; i++) {
-			pstr = da_idx(da, i);
-			if (pstr != NULL) {
-				abspath(pgd->srcdir, pstr, buf); /* XXX check ??? */
-				if (process_template(buf, pgd) == false) {
-					/* failure while processing template */
-					errorf("failed to process '%s'.", buf);
-					rval = 1;
-				}
-			}
-		}
-
-		if (pgd->ac_file != NULL) {
-			pmk_log("\nProcess '%s' for autoconf "
-				"compatibility.\n", pgd->ac_file);
-			if (ac_parse_config(pgd) == false) {
-				/* failure while processing autoconf file */
-				errorf("failed to process autoconf config "
-					"file '%s'.", pgd->ac_file);
+			if (template_main(pgd) == false) {
+				/* an error occured while processing templates */
 				rval = 1;
 			}
-		}
 
-		pmk_log("\nEnd of log\n");
+			pmk_log("\nEnd of log\n");
+		}
+	} else {
+		errorf("failed to open '%s'.", pgd->pmkfile);
+		rval = 1;
 	}
 
+	fclose(fp);
+
 	/* flush and close files */
 	pmk_log_close();
 
@@ -1010,3 +1223,5 @@ int main(int argc, char *argv[]) {
 
 	return(rval);
 }
+
+/* vim: set noexpandtab tabstop=4 softtabstop=4 shiftwidth=4: */
