@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: pmkcfg.sh 1944 2006-12-24 15:01:09Z coudercd $
+# $Id: pmkcfg.sh 1994 2007-05-27 10:54:30Z coudercd $
 
 # Copyright (c) 2003-2004 Damien Couderc
 # Copyright (c) 2004 Martin Reindl
@@ -678,8 +678,10 @@ fi
 # compilation flags #
 ########################################################################
 
+mkf_sed 'configure_input' "Generated by pmkcfg.sh"
 mkf_sed 'CFLAGS' "$CFLAGS $PS_FLAGS"
-mkf_sed 'CLDFLAGS' "$CLDFLAGS $LGEN_FLAGS $LM_FLAGS"
+mkf_sed 'CLDFLAGS' "$CLDFLAGS"
+mkf_sed 'LIBS' "$LGEN_FLAGS $LM_FLAGS"
 
 
 #
