@@ -1,7 +1,7 @@
-/* $Id: pmkinstall.c,v 1.44 2004/12/26 23:42:12 mipsator Exp $ */
+/* $Id: pmkinstall.c 1462 2005-08-21 08:52:06Z mipsator $ */
 
 /*
- * Copyright (c) 2003-2004 Damien Couderc
+ * Copyright (c) 2003-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,12 +42,12 @@
 #include <grp.h>
 #include <limits.h>
 #include <pwd.h>
-#include <stdio.h>
 #include <stdlib.h>
 
 #include "compat/pmk_ctype.h"
 #include "compat/pmk_libgen.h"
 #include "compat/pmk_stdbool.h"
+#include "compat/pmk_stdio.h"
 #include "compat/pmk_string.h"
 #include "compat/pmk_unistd.h"
 #include "common.h"
@@ -57,22 +57,6 @@
 
 /*#define INST_DEBUG	1*/
 
-/* default mode */
-#define DEFAULT_MODE	S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH
-
-/* mode masks */
-#define USR_MASK	S_IRWXU | S_ISUID		/* user */
-#define GRP_MASK	S_IRWXG	| S_ISGID		/* group */
-#define OTH_MASK	S_IRWXO				/* other */
-#define FULL_MASK	USR_MASK | GRP_MASK | OTH_MASK	/* all */
-
-/* perm masks */
-#define R_PERM		S_IRUSR | S_IRGRP | S_IROTH		/* read */
-#define W_PERM		S_IWUSR | S_IWGRP | S_IWOTH		/* write */
-#define X_PERM		S_IXUSR | S_IXGRP | S_IXOTH		/* execute */
-#define S_PERM		S_ISUID | S_ISGID			/* user/group ids */
-#define FULL_PERM	R_PERM | W_PERM | X_PERM | S_PERM	/* all */
-
 
 extern char	*optarg;
 extern int	 optind;
@@ -80,6 +64,8 @@ extern int	 optind;
 extern int	 errno;
 #endif
 
+char		*bsfx = DEFAULT_BACKUP_SFX; /* backup extension */
+
 
 /*
 	strip
@@ -133,7 +119,7 @@ bool symbolic_to_octal_mode(char *mstr, mode_t *pmode) {
 	/* who symbols */
 	while (do_loop == true) {
 #ifdef INST_DEBUG
-debugf("char = '%c'", *pstr);
+		debugf("char = '%c'", *pstr);
 #endif
 		switch (*pstr) {
 			case 'a':
@@ -160,7 +146,7 @@ debugf("char = '%c'", *pstr);
 		}
 		pstr++;
 #ifdef INST_DEBUG
-debugf("mask = %o", mask);
+		debugf("mask = %o", mask);
 #endif
 	}
 
@@ -204,7 +190,7 @@ debugf("mask = %o", mask);
 		pstr++;
 	}
 #ifdef INST_DEBUG
-debugf("perm = %o", perm);
+	debugf("perm = %o", perm);
 #endif
 
 	/* apply operator */
@@ -273,6 +259,155 @@ bool check_mode(char *mstr, mode_t *pmode) {
 	return(true);
 }
 
+/*
+	process owner string
+
+	pstr: owner string
+	puid: storage of uid
+
+	returns: true on success else false
+*/
+
+bool process_owner(char *pstr, uid_t *puid) {
+	struct passwd	*pp = NULL;
+	unsigned long	 ul;
+
+	if (isdigit(*pstr) == 0) {
+		/* user name */
+		pp = getpwnam(pstr);
+		if (pp == NULL) {
+			errorf("invalid user name.");
+			return(false);
+		} else {
+			*puid = pp->pw_uid;
+		}
+	} else {
+		/* uid */
+		if (str_to_ulong(pstr, 10, &ul) == false) {
+			errorf("cannot get numerical value of '%s'.", pstr);
+			return(false);
+		}
+		*puid = (uid_t) ul;
+	}
+
+	return(true);
+}
+
+/*
+	process owner string
+
+	pstr: owner string
+	puid: storage of uid
+
+	returns: true on success else false
+*/
+
+bool process_group(char *pstr, gid_t *pgid) {
+	struct group	*pg = NULL;
+	unsigned long	 ul;
+
+	if (isdigit(*pstr) == 0) {
+		/* group name */
+		pg = getgrnam(pstr);
+		if (pg == NULL) {
+			errorf("invalid group name.");
+			return(false);
+		}
+		*pgid = pg->gr_gid;
+	} else {
+		/* gid */
+		if (str_to_ulong(pstr, 10, &ul) == false) {
+			errorf("cannot get numerical value of '%s'.", pstr);
+			return(false);
+		}
+		*pgid = (gid_t) ul;
+	}
+
+	return(true);
+}
+
+/*
+	create directory
+
+	psrc: source directory
+	buffer: destination buffer
+	bsize: buffer size
+
+	returns: true on success else false
+*/
+
+bool create_directory(char *psrc, char *buffer, size_t bsize) {
+
+#ifdef INST_DEBUG
+	debugf("create dir '%s'", psrc);
+#endif
+
+	/* create path */
+	if (*psrc == CHAR_SEP) {
+	/* absolute path, copy */
+		if (strlcpy_b(buffer, psrc, bsize) == false) {
+			errorf("overflow in directory creation (abs. path).");
+			return(false);
+		}
+	} else {
+		/* relative, getting current directory */
+		if (getcwd(buffer, bsize) == NULL) {
+			errorf("unable to get current directory");
+			return(false);
+		}
+		/* appending path */
+		strlcat(buffer, STR_SEP, bsize); /* no check */
+		if (strlcat_b(buffer, psrc, bsize) == false) {
+			errorf("overflow in directory creation (rel. path).");
+			return(false);
+		}
+	}
+#ifdef INST_DEBUG
+	debugf("dir = '%s'", buffer);
+#endif
+
+	if (makepath(buffer, S_IRWXU | S_IRWXG | S_IRWXO) == false) {
+		errorf("cannot create directory.");
+		return(false);
+	}
+
+	return(true);
+}
+
+/*
+	build destination target
+
+	psrc: source file
+	pdst: destination directory
+	buffer: destination buffer
+	bsize: buffer size
+
+	returns: true on success else false
+*/
+
+bool build_destination(char *psrc, char *pdst, char *buffer, size_t bsize) {
+	char	*pstr;
+
+	strlcpy(buffer, pdst, bsize); /* no check */
+	if (strlcat_b(buffer, STR_SEP, bsize) == false) {
+		errorf("overflow detected in destination.");
+		return(false);
+	}
+
+	pstr = basename(psrc);
+	if (pstr == NULL) {
+		errorf("unable to get basename of source.");
+		return(false);
+	}
+
+	if (strlcat_b(buffer, pstr, bsize) == false) {
+		errorf("overflow detected in destination.");
+		return(false);
+	}
+
+	return(true);
+}
+
 /*
 	usage
 */
@@ -295,10 +430,9 @@ void usage(void) {
 */
 
 int main(int argc, char *argv[]) {
-	struct group	*pg = NULL;
-	struct passwd	*pp = NULL;
 	struct stat	 sb;
 	bool		 create_dir = false,
+			 do_backup = false,
 			 do_chown = false,
 			 do_strip = false,
 			 go_exit = false;
@@ -306,8 +440,8 @@ int main(int argc, char *argv[]) {
 			*ostr = NULL,
 			*src,
 			*dst,
-			*pstr,
-			 dir[MAXPATHLEN];
+			 dir[MAXPATHLEN],
+			 backup[MAXPATHLEN];
 	gid_t		 gid = (gid_t) -1;
 	int		 chr;
 	mode_t		 mode = DEFAULT_MODE,
@@ -315,18 +449,23 @@ int main(int argc, char *argv[]) {
 	uid_t		 uid = (uid_t) -1;
 	unsigned int	 src_idx,
 			 last_idx;
-	unsigned long	 ul;
-	
+
 	while (go_exit == false) {
-		chr = getopt(argc, argv, "bcdg:hm:o:stv");
+		chr = getopt(argc, argv, "B:bcdg:hm:o:stv");
 		if (chr == -1) {
 			go_exit = true;
 		} else {
 			switch (chr) {
+				case 'B' :
+					bsfx = optarg;
+#ifdef INST_DEBUG
+					debugf("backup suffix = %s", bsfx);
+#endif
+					break;
+
 				case 'b' :
 					/* backup */
-					/* XXX TODO */
-					errorf("-b option has not been implemented yet");
+					do_backup = true;
 					break;
 
 				case 'c' :
@@ -342,7 +481,7 @@ int main(int argc, char *argv[]) {
 					/* specify group */
 					gstr = optarg;
 #ifdef INST_DEBUG
-debugf("gstr = %s", gstr);
+					debugf("gstr = %s", gstr);
 #endif
 					break;
 
@@ -351,7 +490,7 @@ debugf("gstr = %s", gstr);
 					if (check_mode(optarg, &mode) == false)
 						exit(EXIT_FAILURE);
 #ifdef INST_DEBUG
-debugf("mode = %o", mode);
+					debugf("mode = %o", mode);
 #endif
 					break;
 
@@ -395,57 +534,25 @@ debugf("mode = %o", mode);
 
 	/* check if an owner has been provided */
 	if (ostr != NULL) {
-		if (isdigit(*ostr) == 0) {
-			/* user name */
-			pp = getpwnam(ostr);
-			if (pp == NULL) {
-				errorf("invalid user name.");
-				exit(EXIT_FAILURE);
-			} else {
-				uid = pp->pw_uid;
-#ifdef INST_DEBUG
-debugf("uid = %d", uid);
-#endif
-			}
-		} else {
-			/* uid */
-			if (str_to_ulong(ostr, 10, &ul) == false) {
-				errorf("cannot get numerical value of '%s'.", ostr);
-				exit(EXIT_FAILURE);
-			}
-			uid = (uid_t) ul;
-#ifdef INST_DEBUG
-debugf("uid = %d", uid);
-#endif
+		if (process_owner(ostr, &uid) == false) {
+			/* error message already done */
+			exit(EXIT_FAILURE);
 		}
+#ifdef INST_DEBUG
+		debugf("uid = %d", uid);
+#endif
 		do_chown = true;
 	}
 
 	/* if a group has been provided */
 	if (gstr != NULL) {
-		if (isdigit(*gstr) == 0) {
-			/* group name */
-			pg = getgrnam(gstr);
-			if (pg == NULL) {
-				errorf("invalid group name.");
-				exit(EXIT_FAILURE);
-			} else {
-				gid = pg->gr_gid;
-#ifdef INST_DEBUG
-debugf("gid = %d", gid);
-#endif
-			}
-		} else {
-			/* gid */
-			if (str_to_ulong(gstr, 10, &ul) == false) {
-				errorf("cannot get numerical value of '%s'.", gstr);
-				exit(EXIT_FAILURE);
-			}
-			gid = (gid_t) ul;
-#ifdef INST_DEBUG
-debugf("gid = %d", gid);
-#endif
+		if (process_group(gstr, &gid) == false) {
+			/* error message already done */
+			exit(EXIT_FAILURE);
 		}
+#ifdef INST_DEBUG
+		debugf("gid = %d", gid);
+#endif
 		do_chown = true;
 	}
 
@@ -461,59 +568,26 @@ debugf("gid = %d", gid);
 		src = argv[src_idx];
 
 		if (create_dir == true) {
-			/*
-				directory creation
-			*/
-
-#ifdef INST_DEBUG
-debugf("create dir '%s'", src);
-#endif
-
-			/* create path */
-			if (*src == CHAR_SEP) {
-			/* absolute path, copy */
-				if (strlcpy_b(dir, src, sizeof(dir)) == false) {
-					errorf("overflow detected in directory creation (abs. path).");
-					exit(EXIT_FAILURE);
-				}
-			} else {
-				/* relative, getting current directory */
-				if (getcwd(dir, sizeof(dir)) == NULL) {
-					errorf("unable to get current directory");
-					exit(EXIT_FAILURE);
-				}
-				/* appending path */
-				strlcat(dir, STR_SEP, sizeof(dir)); /* no check */
-				if (strlcat_b(dir, src, sizeof(dir)) == false) {
-					errorf("overflow detected in directory creation (rel. path).");
-					exit(EXIT_FAILURE);
-				}
-			}
-#ifdef INST_DEBUG
-debugf("dir = '%s'", dir);
-#endif
-
-			if (makepath(dir, S_IRWXU | S_IRWXG | S_IRWXO) == false) {
-				errorf("cannot create directory.");
+			/* create directory */
+			if (create_directory(src, dir, sizeof(dir)) == false) {
+				/* error message already done */
 				exit(EXIT_FAILURE);
 			}
 
 			/* set dst for further operations */
 			dst = dir;
 		} else {
-			/*
-				install file
-			*/
+			/* install file	*/
 
 #ifdef INST_DEBUG
-debugf("process install of '%s'", src);
+			debugf("process install of '%s'", src);
 #endif
 
 			/*  set dst */
 			dst = argv[last_idx];
 
 #ifdef INST_DEBUG
-debugf("initial dst = '%s'", dst);
+			debugf("initial dst = '%s'", dst);
 #endif
 
 			/* check if destination is a directory */
@@ -521,31 +595,42 @@ debugf("initial dst = '%s'", dst);
 				/* many checks to do (is a directory, etc ...) */
 				tmode = sb.st_mode & S_IFDIR;
 				if (tmode == 0) {
-					/* not a directory, XXX backup ? */
+					/* not a directory */
 					unlink(dst);
 				} else {
-					strlcpy(dir, dst, sizeof(dir)); /* no check */
-					if (strlcat_b(dir, STR_SEP, sizeof(dir)) == false) {
-						errorf("overflow detected in destination.");
+					/* build destination */
+					if (build_destination(src, dst, dir,
+							sizeof(dir)) == false) {
+						/* error message already done */
 						exit(EXIT_FAILURE);
 					}
 
-					pstr = basename(src);
-					if (pstr == NULL) {
-						errorf("unable to get basename of source.");
-						exit(EXIT_FAILURE);
-					}
-
-					if (strlcat_b(dir, pstr, sizeof(dir)) == false) {
-						errorf("overflow detected in destination.");
-						exit(EXIT_FAILURE);
-					}
+					/* set dst for further operations */
 					dst = dir;
 				}
 			}
 
+
+			/* backup, check dst again as it could have been modified */
+			if ((do_backup == true) && (stat(dst, &sb) == 0)) {
 #ifdef INST_DEBUG
-debugf("copy to '%s'", dst);
+				debugf("backup of '%s'", dst);
+#endif
+				if (snprintf_b(backup, sizeof(backup),
+						"%s%s", dst, bsfx) == false) {
+					errorf("failed to generate backup name for '%s': %s.",
+							dst, strerror(errno));
+					/* XXX exit ? */
+				}
+
+				if (rename(dst, backup) != 0) {
+					errorf("failed to backup %s: %s.",
+							dst, strerror(errno));
+				}
+			}
+
+#ifdef INST_DEBUG
+			debugf("copy to '%s'", dst);
 #endif
 			/* copy file */
 			if (fcopy(src, dst, mode) == false) {
@@ -562,7 +647,7 @@ debugf("copy to '%s'", dst);
 		/* change owner and group */
 		if (do_chown == true) {
 #ifdef INST_DEBUG
-debugf("doing chown('%s', %d, %d)", dst, uid, gid);
+			debugf("doing chown('%s', %d, %d)", dst, uid, gid);
 #endif
 			if (chown(dst, uid, gid) != 0) {
 				errorf("chown failed : %s.", strerror(errno));
@@ -573,7 +658,7 @@ debugf("doing chown('%s', %d, %d)", dst, uid, gid);
 		/* change perms (must follow chown that can change perms) */
 		if (chmod(dst, mode) == -1) {
 #ifdef INST_DEBUG
-debugf("chmod('%s', %o)", dst, mode);
+			debugf("chmod('%s', %o)", dst, mode);
 #endif
 			errorf("chmod failed : %s.", strerror(errno));
 			exit(EXIT_FAILURE);
