@@ -1,7 +1,7 @@
-/* $Id: pmkpc.c,v 1.22.6.1 2005/04/25 20:30:26 mipsator Exp $ */
+/* $Id: pmkpc.c 1952 2007-02-15 19:01:17Z coudercd $ */
 
 /*
- * Copyright (c) 2004 Damien Couderc
+ * Copyright (c) 2004-2005 Damien Couderc
  * Copyright (c) 2004 Xavier Santolaria <xavier@santolaria.net>
  * All rights reserved.
  *
@@ -36,9 +36,9 @@
 
 
 #include <errno.h>
-#include <stdio.h>
 #include <stdlib.h>
 
+#include "compat/pmk_stdio.h"
 #include "compat/pmk_string.h"
 #include "cfgtool.h"
 #include "common.h"
@@ -52,42 +52,60 @@
 
 /*#define PMKPC_DEBUG	1*/
 
+
 extern char	*optarg;
 extern int	 optind;
 
+
+/**********
+ variables
+************************************************************************/
+
 pcopt	pcoptions[] = {
-		{"version",			false,	PMKPC_OPT_VERSION,		NULL},
-		{"atleast-pkgconfig-version",	true,	PMKPC_OPT_ATLPKGVERS,		PC_USAGE_VERSION},
-		{"exists",			false,	PMKPC_OPT_EXISTS,		NULL},
-		{"list-all",			false,	PMKPC_OPT_LISTALL,		NULL},
-		{"uninstalled",			false,	PMKPC_OPT_UNINST,		NULL},
-		{"debug",			false,	PMKPC_OPT_DEBUG,		NULL},
-		{"help",			false,	PMKPC_OPT_HELP,			NULL},
-		{"usage",			false,	PMKPC_OPT_USAGE,		NULL},
-		{"modversion",			false,	PMKPC_OPT_MODVERS,		NULL},
-		{"atleast-version",		true,	PMKPC_OPT_ATLVERS,		PC_USAGE_VERSION},
-		{"exact-version",		true,	PMKPC_OPT_EXTVERS,		PC_USAGE_VERSION},
-		{"max-version",			true,	PMKPC_OPT_MAXVERS,		PC_USAGE_VERSION},
-		{"cflags",			false,	PMKPC_OPT_CFLAGS,		NULL},
-		{"cflags-only-I",		false,	PMKPC_OPT_CFLAGS_ONLY_PATH,	NULL},
-		{"cflags-only-other",		false,	PMKPC_OPT_CFLAGS_ONLY_OTHER,	NULL},
-		{"libs",			false,	PMKPC_OPT_LIBS, 		NULL},
-		{"libs-only-l",			false,	PMKPC_OPT_LIBS_ONLY_LIB,	NULL},
-		{"libs-only-L",			false,	PMKPC_OPT_LIBS_ONLY_PATH,	NULL},
-		{"libs-only-other",		false,	PMKPC_OPT_LIBS_ONLY_OTHER,	NULL},
-		{"variable",			true,	PMKPC_OPT_VAR,			PC_USAGE_VARNAME},
-		{"define-variable",		true,	PMKPC_OPT_VAR_DEF,		PC_USAGE_VARVAL},
-		{"print-errors",		false,	PMKPC_OPT_VAR_PRNT,		NULL},
-		{"silence-errors",		false,	PMKPC_OPT_VAR_SILC,		NULL},
-		{"errors-to-stdout",		false,	PMKPC_OPT_VAR_STDO,		NULL},
+		{"version",						false,	PMKPC_OPT_VERSION,				NULL},
+		{"atleast-pkgconfig-version",	true,	PMKPC_OPT_ATLPKGVERS,			PC_USAGE_VERSION},
+		{"exists",						false,	PMKPC_OPT_EXISTS,				NULL},
+		{"list-all",					false,	PMKPC_OPT_LISTALL,				NULL},
+		{"uninstalled",					false,	PMKPC_OPT_UNINST,				NULL},
+		{"debug",						false,	PMKPC_OPT_DEBUG,				NULL},
+		{"help",						false,	PMKPC_OPT_HELP,					NULL},
+		{"usage",						false,	PMKPC_OPT_USAGE,				NULL},
+		{"modversion",					false,	PMKPC_OPT_MODVERS,				NULL},
+		{"atleast-version",				true,	PMKPC_OPT_ATLVERS,				PC_USAGE_VERSION},
+		{"exact-version",				true,	PMKPC_OPT_EXTVERS,				PC_USAGE_VERSION},
+		{"max-version",					true,	PMKPC_OPT_MAXVERS,				PC_USAGE_VERSION},
+		{"cflags",						false,	PMKPC_OPT_CFLAGS,				NULL},
+		{"cflags-only-I",				false,	PMKPC_OPT_CFLAGS_ONLY_PATH,		NULL},
+		{"cflags-only-other",			false,	PMKPC_OPT_CFLAGS_ONLY_OTHER,	NULL},
+		{"libs",						false,	PMKPC_OPT_LIBS, 				NULL},
+		{"libs-only-l",					false,	PMKPC_OPT_LIBS_ONLY_LIB,		NULL},
+		{"libs-only-L",					false,	PMKPC_OPT_LIBS_ONLY_PATH,		NULL},
+		{"libs-only-other",				false,	PMKPC_OPT_LIBS_ONLY_OTHER,		NULL},
+		{"variable",					true,	PMKPC_OPT_VAR,					PC_USAGE_VARNAME},
+		{"define-variable",				true,	PMKPC_OPT_VAR_DEF,				PC_USAGE_VARVAL},
+		{"print-errors",				false,	PMKPC_OPT_VAR_PRNT,				NULL},
+		{"silence-errors",				false,	PMKPC_OPT_VAR_SILC,				NULL},
+		{"errors-to-stdout",			false,	PMKPC_OPT_VAR_STDO,				NULL},
 };
 size_t	nbpcopt = sizeof(pcoptions) / sizeof(pcopt);
 
-/*
+
+/**********
+ functions
+************************************************************************/
+
+/***************
+ optcell_init()
+
+ DESCR
 	init optcell structure
 
-	return : optcell structure
-*/
+ IN
+	-
+
+ OUT
+	optcell structure
+************************************************************************/
 
 optcell *optcell_init(void) {
 	optcell	*poc;
@@ -104,51 +122,63 @@ optcell *optcell_init(void) {
 	return(poc);
 }
 
-/*
+
+/******************
+ optcell_destroy()
+
+ DESCR
 	destroy optcell structure
 
+ IN
 	poc : optcell structure
 
-	return : -
-*/
+ OUT
+	-
+************************************************************************/
 
 void optcell_destroy(optcell *poc) {
 	free(poc); /* no more to free */
 }
 
-/*
+
+/***********
+ pcgetopt()
+
+ DESCR
 	process command line options
 
-	ac : argument number
-	av : argument array
-	poc : optcell structure
+ IN
+	ac :	argument number
+	av :	argument array
+	poc :	optcell structure
 
-	return : boolean
-*/
+ OUT
+	boolean
+************************************************************************/
 
 bool pcgetopt(unsigned int ac, char *av[], optcell *poc) {
-	static char	 buf[PMKPC_MAX_OPT_SIZE];
-	char		*opt,
-			*pstr;
-	size_t		 l;
+	static char		 buf[PMKPC_MAX_OPT_SIZE];
+	char			*opt,
+					*pstr;
+	size_t			 l;
 	unsigned int	 i,
-			 idx;
+					 idx;
 
 	idx = poc->idx;
 
 	if (idx >= ac) {
 #ifdef PMKPC_DEBUG
-debugf("{pcgetopt} ac = %d", ac);
+		debugf("{pcgetopt} ac = %d", ac);
 #endif
 		/* no more args */
 		return(false);
 	}
 
 	opt = av[idx];
-	
+
 	if ((opt[0] != '-') || (opt[1] != '-')) {
 #ifdef PMKPC_DEBUG
-debugf("{pcgetopt} full opt = '%s'", opt);
+		debugf("{pcgetopt} full opt = '%s'", opt);
 #endif
 		/* not an option */
 		poc->arg = opt;
@@ -157,7 +187,7 @@ debugf("{pcgetopt} full opt = '%s'", opt);
 		/* skip leading "--" */
 		opt = opt + 2;
 #ifdef PMKPC_DEBUG
-debugf("{pcgetopt} opt = '%s'", opt);
+		debugf("{pcgetopt} opt = '%s'", opt);
 #endif
 
 		/* option have argument ? */
@@ -178,7 +208,7 @@ debugf("{pcgetopt} opt = '%s'", opt);
 			pstr++;
 			poc->arg = pstr;
 #ifdef PMKPC_DEBUG
-debugf("{pcgetopt} opt arg = '%s'", poc->arg);
+		debugf("{pcgetopt} opt arg = '%s'", poc->arg);
 #endif
 
 			/* set opt back to the beginning of the option */
@@ -210,20 +240,30 @@ debugf("{pcgetopt} idx = %d", idx);
 	return(true);
 }
 
-/*
+
+/***********
+ list_all()
+
+ DESCR
 	list all packages
-*/
+
+ IN
+	ppd :	XXX
+
+ OUT
+	boolean
+************************************************************************/
 
 bool list_all(pkgdata *ppd) {
-	bool		 rval = true;
-	char		 *mod,
-			 *pcfile,
-			**descr,
-			  fmt[32];
-	hkeys		 *phk;
-	pkgcell		 *ppc;
-	size_t		  len,
-			  maxs = 0;
+	bool			  rval = true;
+	char			 *mod,
+					 *pcfile,
+					**descr,
+					  fmt[32];
+	hkeys			 *phk;
+	pkgcell			 *ppc;
+	size_t			  len,
+					  maxs = 0;
 	unsigned int	  i;
 
 	phk = hash_keys(ppd->files);
@@ -274,9 +314,19 @@ bool list_all(pkgdata *ppd) {
 	return(true);
 }
 
-/*
-	clean
-*/
+
+/********
+ clean()
+
+ DESCR
+	clean XXX
+
+ IN
+	p_cd :	XXX
+
+ OUT
+	-
+************************************************************************/
 
 void clean(pcdata *p_cd) {
 	/* clean data */
@@ -291,15 +341,25 @@ void clean(pcdata *p_cd) {
 	}
 }
 
-/*
+
+/********
+ usage()
+
+ DESCR
  	pmkpc(1) usage
-*/
+
+ IN
+	-
+
+ OUT
+	-
+************************************************************************/
 
 void usage(void) {
-	char		option[MAXPATHLEN];
-	size_t		alen,	/* alignement length */
-			olen,	/* option length */
-			cursor;
+	char			option[MAXPATHLEN];
+	size_t			alen,	/* alignement length */
+					olen,	/* option length */
+					cursor;
 	unsigned int	i;
 
 	alen = strlen(PMKPC_USAGE_ALIGN);
@@ -336,40 +396,44 @@ void usage(void) {
 	fprintf(stderr, "\n");
 }
 
-/*
-	main
-*/
+
+/*******
+ main()
+
+ DESCR
+	main loop
+************************************************************************/
 
 int main(int argc, char *argv[]) {
-	FILE		*fp;
-	bool		 opt_version = false,
-			 opt_atl_vers = false,
-			 opt_modvers = false,
-			 opt_cmp_modvers = false,
-			 opt_exists = false,
-			 opt_listall = false,
-			 opt_cflags = false,
-			 opt_libs = false;
-	cfgtcell	*pcc = NULL;
-	cfgtdata	*pcd = NULL;
-	char		*mod,
-			*bpath,
-			*opt,
-			*pc_path,
-			*pstr,
-			*mvers = NULL,
-			*pvers = NULL,
-			 cmp_type = CHAR_EOS,
-			 pc_cmd[MAXPATHLEN],
-			 pc_buf[MAXPATHLEN],
-			 buf[TMP_BUF_LEN];
-	int		 r;
-	optcell		*poc;
-	pcdata		 gdata;
-	pkgcell		*ppc;
+	FILE			*fp;
+	bool			 opt_version = false,
+					 opt_atl_vers = false,
+					 opt_modvers = false,
+					 opt_cmp_modvers = false,
+					 opt_exists = false,
+					 opt_listall = false,
+					 opt_cflags = false,
+					 opt_libs = false;
+	cfgtcell		*pcc = NULL;
+	cfgtdata		*pcd = NULL;
+	char			*mod,
+					*bpath,
+					*opt,
+					*pc_path,
+					*pstr,
+					*mvers = NULL,
+					*pvers = NULL,
+					 cmp_type = CHAR_EOS,
+					 pc_cmd[MAXPATHLEN],
+					 pc_buf[MAXPATHLEN],
+					 buf[TMP_BUF_LEN];
+	int				 r;
+	optcell			*poc;
+	pcdata			 gdata;
+	pkgcell			*ppc;
 	unsigned int	 i,
-			 cflags_opts = 0,
-			 libs_opts = 0;
+					 cflags_opts = 0,
+					 libs_opts = 0;
 
 	poc = optcell_init();
 	if (poc == NULL) {
@@ -403,7 +467,7 @@ int main(int argc, char *argv[]) {
 	while (poc->idx != (unsigned int) argc) {
 		if (pcgetopt(argc, argv, poc) == true) {
 #ifdef PMKPC_DEBUG
-debugf("{main} id = %d", poc->id);
+			debugf("{main} id = %d", poc->id);
 #endif
 			/* found an option */
 			switch (poc->id) {
@@ -606,16 +670,16 @@ debugf("{main} parsed '%s'", PREMAKE_CONFIG_PATH);
 
 
 #ifdef PMKPC_DEBUG
-debugf("{main} usize = '%d'", da_usize(gdata.pda));
+	debugf("{main} usize = '%d'", da_usize(gdata.pda));
 #endif
 	for (i = 0 ; i < da_usize(gdata.pda) ; i++) {
 		mod = da_idx(gdata.pda, i);
 #ifdef PMKPC_DEBUG
-debugf("{main} mod = '%s' (i = %d)", mod, i);
+		debugf("{main} mod = '%s' (i = %d)", mod, i);
 #endif
 		if (pkg_mod_exists(gdata.ppd, mod) == true) {
 #ifdef PMKPC_DEBUG
-debugf("{main} module '%s' found", mod);
+		debugf("{main} module '%s' found", mod);
 #endif
 			ppc = pkg_cell_add(gdata.ppd, mod); /* ppc is part of ppd, don't destroy */
 
@@ -677,14 +741,23 @@ debugf("{main} module '%s' found", mod);
 				}
 
 				if (opt_cflags == true) {
-					if (cflags_opts == 0)
+					/* display cflags */
+					if (cflags_opts == 0) {
 						cflags_opts = PKGCFG_CFLAGS_ALL;
+					}
 
 					printf("%s", pkg_get_cflags_adv(gdata.ppd, cflags_opts));
 				}
 				if (opt_libs == true) {
-					if (libs_opts == 0)
+					/* if cflags have been displayed, add a space */
+					if (opt_cflags == true) {
+						printf(" ");
+					}
+
+					/* display libs */
+					if (libs_opts == 0) {
 						libs_opts = PKGCFG_LIBS_ALL;
+					}
 
 					printf("%s", pkg_get_libs_adv(gdata.ppd, libs_opts));
 				}
@@ -692,7 +765,7 @@ debugf("{main} module '%s' found", mod);
 			}
 		} else {
 #ifdef PMKPC_DEBUG
-debugf("module not found");
+			debugf("module not found");
 #endif
 
 			/* init config tool data if needed */
@@ -726,7 +799,7 @@ debugf("module not found");
 			if (get_file_path(pc_cmd, bpath, pc_buf, sizeof(pc_buf)) == true) {
 				/* use CHECK_CONFIG */
 #ifdef PMKPC_DEBUG
-debugf("Found alternative '%s' tool.", pc_cmd);
+				debugf("Found alternative '%s' tool.", pc_cmd);
 #endif
 
 				if (opt_modvers == true) {
