@@ -1,4 +1,4 @@
-/* $Id: pmkpc.h,v 1.8 2004/12/23 17:36:52 mipsator Exp $ */
+/* $Id: pmkpc.h 1405 2005-04-25 21:22:55Z mipsator $ */
 
 /*
  * Copyright (c) 2004 Damien Couderc
@@ -38,6 +38,10 @@
 #define _PMKPC_H_
 
 
+/**********
+ constants
+************************************************************************/
+
 /* options id */
 #define PMKPC_OPT_UNKNOWN		0
 #define PMKPC_OPT_VERSION		1
@@ -90,6 +94,11 @@
 #define PC_USAGE_VARNAME	"VARIABLENAME"
 #define PC_USAGE_VARVAL		"VARIABLENAME=VARIABLEVALUE"
 
+
+/*********************************
+ types and structures definitions
+************************************************************************/
+
 typedef struct {
 	char		*name;
 	bool		 arg;
@@ -111,6 +120,10 @@ typedef struct {
 } pcdata;
 
 
+/********************
+ function prototypes
+************************************************************************/
+
 bool	 list_all(pkgdata *);
 bool	 pcgetopt(unsigned int, char **, optcell *);
 optcell	*optcell_init(void);
