@@ -1,4 +1,4 @@
-/* $Id: cpuid_test.c,v 1.12 2004/12/26 21:07:03 xsa Exp $ */
+/* $Id: cpuid_test.c 1442 2005-08-02 20:36:21Z mipsator $ */
 
 /* Public Domain */
 
@@ -9,14 +9,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "../common.h"
-#include "../compat/compat.h"
 #include "../detect_cpu.h"
 #include "../detect_cpu_asm.h"
-#include "../dynarray.h"
 #include "../hash.h"
 #include "../parse.h"
-#include "../pmk_obj.h"
 
 #define PMKCPU_DATA	"../data/pmkcpu.dat"
 
