@@ -1,4 +1,4 @@
-/* $Id: gen_comp_test.c,v 1.4 2004/12/26 20:59:45 xsa Exp $ */
+/* $Id: gen_comp_test.c 1442 2005-08-02 20:36:21Z mipsator $ */
 
 /* Public Domain */
 
@@ -9,13 +9,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "../common.h"
-#include "../compat/compat.h"
 #include "../detect.h"
-#include "../dynarray.h"
-#include "../hash.h"
 #include "../parse.h"
-#include "../pmk_obj.h"
 
 int main(void) {
 	FILE		*fp;
@@ -33,7 +28,7 @@ int main(void) {
 	gen_test_file(fp, cdata);
 
 	fclose(fp);
-	
+
 	compdata_destroy(cdata);
 
 	return(0);
