@@ -1,14 +1,13 @@
-/* $Id: hash_test.c,v 1.18 2004/12/26 21:05:32 xsa Exp $ */
+/* $Id: hash_test.c 1463 2005-08-21 09:40:11Z mipsator $ */
 
 /* Public Domain license */
 
 /* hash table test */
 
-#include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 
-#include "../compat/compat.h"
+#include "../compat/pmk_stdio.h"
 #include "../compat/pmk_string.h"
 #include "../hash.h"
 
@@ -126,7 +125,7 @@ int main(void) {
 	}
 
 	printf("Testing destroy\n");
-	n = hash_destroy(hp);	
+	n = hash_destroy(hp);
 	printf("Removed %d key(s)\n", n);
 
 	return(0);
