@@ -1,4 +1,4 @@
-/* $Id: pkgconfig_test.c,v 1.8 2004/12/27 11:24:34 xsa Exp $ */
+/* $Id: pkgconfig_test.c 1463 2005-08-21 09:40:11Z mipsator $ */
 
 /* Public Domain */
 
@@ -7,18 +7,12 @@
 /*#define PKGCFG_DEBUG	1*/
 /*#define HASH_DEBUG	1*/
 
-#include <stdio.h>
 #include <stdlib.h>
 
-#include "../compat/compat.h"
 #include "../compat/pmk_stdbool.h"
+#include "../compat/pmk_stdio.h"
 #include "../common.h"
-#include "../dynarray.h"
-#include "../hash.h"
-#include "../parse.h"
 #include "../pkgconfig.h"
-#include "../pmk_obj.h"
-#include "../premake.h"
 
 int main(int argc, char *argv[]) {
 	char	*mod;
